/**
 * @fileOverview responsible for the managing the users assigned to the study application. Depends on dynamicGeneration
 *               and RAMS_Tools.
 * @version 2.3.0
 */

/* globals RAMS, consoleLoggingEnabled, spAppData, siteUrl, assignedID, restPath, currentUser */
/* globals sendREST, getFacilityNameById, logError, textCellHelper, headerBuilder, sideColumnBuilder */
/* globals tableHeaderHelper, progressBuilder, buttonCellHelper, PARENTSTUDYPI */
/* globals PARENTSTUDYCOPI, PRINCIPALINVESTIGATOR, radioFieldHelper, linkCellHelper, buildSelecterHelper */
/* globals smcPlusIncumbents, insertStudyMember, restUrl, generateAssignedId, selectFieldHelper */
/* globals textFieldHelper, confirm, SP, selectCellHelper, insertWarning */
/* globals textareaCellHelper, ramsUsers, targetStage, requestDigest, LOCALSITEINVESTIGATOR, COPRINCIPALINVESTIGATOR */
var currentAttachmentID;
var userObject = {};
var localSiteAppObject = {};

function refreshDisplay(refreshType) {
   var row, container, pageHeader, sideWell, pageFooter, headerArea, attachments;

   $('#refresh-section').remove();
   // $('div#insert').attr("class", "container");
   row = document.createElement("div");
   row.setAttribute("class", "row");
   row.setAttribute("id", "refresh-section");
   $('div#insert').append(row);

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-9");
   row.appendChild(container);

   pageHeader = document.createElement("div");
   pageHeader.setAttribute("class", "page-header");
   pageHeader.setAttribute("tabindex", "0");
   container.appendChild(pageHeader);

   headerArea = $('div.page-header');
   headerArea.empty();

   if (refreshType == "Facility") {
      headerBuilder("Sites", headerArea);
      sendREST("GET", siteUrl.concat("_api/Web/Lists/getByTitle('facilities')/Items"), null, function(groupJSON) {
         var currentGroups = groupJSON.d.results;
         var facility, userGroupID, localSiteLiaisonID;

         // loop through SharePoint facility groups
         // Set userObject facility details by the facility list
         for (var i = 0; i < currentGroups.length; i++) {
            facility = currentGroups[i].Title;
            userGroupID = currentGroups[i].UsersId;
            localSiteLiaisonID = currentGroups[i].FacilityLiaisonId;

            userObject[facility] = {};
            userObject[facility].userGroupID = userGroupID;
            userObject[facility].groupID = currentGroups[i].Id;
            userObject[facility].localSiteLiaisonID = localSiteLiaisonID;

            // use AJAX calls to add user and liaison data for each facility to global object
            getUsers(userGroupID, facility);
            getLiaison(localSiteLiaisonID, facility);
         }

         grabLocalSiteApplications(assignedID); // save local site app data to global object
         container.appendChild(facilityDisplay()); // build local site applications page

         pageFooter = document.createElement("div");
         pageFooter.setAttribute("class", "progress-container");
         container.appendChild(pageFooter);
         pageFooter = document.createElement("div");
         pageFooter.setAttribute("class", "button-container");
         container.appendChild(pageFooter);

         progressBuilder();
      });
   } else if (refreshType == "Personnel") {
      var AOMember = aoMemberCheck();

      headerBuilder("Personnel", headerArea);

      // NOT USED facilityId
      // var facilityId = spAppData.FacilitiesLookupId;
      var facilityName = getFacilityNameById(spAppData.FacilitiesLookupId);
      sendREST(
         "GET",
         siteUrl.concat("_api/Web/sitegroups" + "?$filter=startswith(Title, '" + facilityName + "') eq true"),
         null,
         function(groupJSON) {
            var currentGroups = groupJSON.d.results;
            var siteUsersGroupID = -1;

            for (var i = 0; i < currentGroups.length; i++) { // find the site users' group for the user's site
               // This does not read the facility question. Should it?
               if (currentGroups[i].Title.indexOf("Users") > -1) {
                  siteUsersGroupID = currentGroups[i].Id;
               }
            }

            // perform REST calls if we found site users group
            if (siteUsersGroupID > 0) { // assuming user belongs to only one site
               sendREST(
                  "GET",
                  siteUrl.concat("_api/Web/SiteGroups(" + siteUsersGroupID + ")/Users"),
                  null,
                  function(siteUsers) {
                     sendREST(
                        "GET",
                        restPath.concat("study/" + assignedID + "/members"),
                        null,
                        function(studyMembers) {
                           container.appendChild(personnelDisplay(studyMembers, siteUsers));
                           var rows = document.getElementsByName("SPU");
                           for (var i = 0; i < rows.length; i++) {
                              rows[i].style.display = "none";
                           }

                           // container.appendChild(editPersonnelDisplay());

                           pageFooter = document.createElement("div");
                           pageFooter.setAttribute("class", "progress-container");
                           container.appendChild(pageFooter);
                           pageFooter = document.createElement("div");
                           pageFooter.setAttribute("class", "button-container");
                           container.appendChild(pageFooter);

                           if ((AOMember === true)
                              || (currentUser.Id == spAppData[RAMS.Environment.SharePoint.List.Application.PrincipalInvestigatorId])
                              || (currentUser.Id == spAppData[RAMS.Environment.SharePoint.List.Application.CoPrincipalInvestigatorId])) {
                              progressBuilder(true);
                              $("[value='Save Progress']").attr("onclick", "insertPerson()");
                           } else {
                              progressBuilder();
                           }
                           // $("[value='Save Progress']").attr("onkeypress", "insertPerson()");
                        });
                  });
            }
         });
   } else if (refreshType == "Files") {
      headerBuilder("Attachments", headerArea);

      sendREST("GET", siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('Shared%20Documents/" + assignedID
         + "/Study%20Team')/Files?$expand=ListItemAllFields"), null, function(returnedFiles) {
         // attachments = filterChecklists(returnedFiles.d.results, "-RAMS-");
         attachments = returnedFiles.d.results;
         container.appendChild(attachmentsDisplay(attachments));

         pageFooter = document.createElement("div");
         pageFooter.setAttribute("class", "progress-container");
         container.appendChild(pageFooter);
         pageFooter = document.createElement("div");
         pageFooter.setAttribute("class", "button-container");
         container.appendChild(pageFooter);

         progressBuilder();
      });
   }

   sideWell = document.createElement("div");
   sideWell.setAttribute("class", "col-xs-3 toolbar");
   row.appendChild(sideWell);

   sideColumnBuilder(null);
}

/* add the users found to the global object userObject */
function addUserArray(data, facility) {
   var userArray = [];
   var user = data.d.results;

   for (var i = 0; i < user.length; i++) {
      userArray.push(user[i].Title);
      userObject[facility][user[i].Title] = {};
      userObject[facility][user[i].Title].id = user[i].Id;
   }
   userObject[facility].users = userArray;
}

/* return the JSON of the users belonging to a site group */
function getUsers(groupID, facility) { // replace committee with facility
   return $.ajax({
      async : false,
      type : 'GET',
      url : siteUrl.concat("_api/Web/sitegroups(" + groupID + ")/users"),
      headers : {
         "accept" : "application/json;odata=verbose"
      },
      dataType : 'json',
      success : function(json) {
         addUserArray(json, facility);
      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError((siteUrl.concat("_api/Web/sitegroups(" + groupID + ")/users")), errorThrown, textStatus, JSON
            .stringify(XMLHttpRequest));
      }
   });
}

/* add the local site liaison found to the global object userObject */
function addLiaison(data, facility) {
   /*
    * TAR data is a user object var user = data.d.results; userObject[facility].localSiteLiaison = user[0].Title;
    */
   var user = data.d;
   userObject[facility].localSiteLiaison = user.Title;

}

/* return the JSON of the local site liaison belonging to a site group */
function getLiaison(localSiteLiaisonID, facility) {
   /*
    * Facility Local Site Liaison is a user, not a group. Leaving this in case it becomes a group later. return $.ajax({
    * async: false, type: 'GET', url: siteUrl.concat("_api/Web/sitegroups("+localSiteLiaisonID+")/users"), headers:
    * {"accept": "application/json;odata=verbose"}, dataType: 'json', success: function(json){ addLiaison(json,
    * facility); } });
    */
   return $.ajax({
      async : false,
      type : 'GET',
      url : siteUrl.concat("_api/Web/siteusers/getbyid(" + localSiteLiaisonID + ")"),
      headers : {
         "accept" : "application/json;odata=verbose"
      },
      dataType : 'json',
      success : function(json) {
         addLiaison(json, facility);
      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError((siteUrl.concat("_api/Web/sitegroups(" + localSiteLiaisonID + ")/users")), errorThrown, textStatus,
            JSON.stringify(XMLHttpRequest));
      }
   });
}

function personnelDisplay(studyMembers, siteUsers) {
   var container, table, tableDiv, tbody, trow, buttonID, buttonID_Remove;
   // no longer used, but remains in the comments, so leaving it in and commenting it out for now.
   // var skipFlag, cell;

   // var selectUsers = [];
   // var tempList = ramsUsers.values();
   // ******************************** NEED THIS TO WORK OFF OF A SP USER GROUP

   // //not including users who are already in the table and are active
   // for(var i=0; i<tableUsers.length; i++){
   // skipFlag = false;
   // for(var j=0; j<returnedMembers.length; j++){
   // if((tableUsers[i].displayName == returnedMembers[j].user.displayName) && (returnedMembers[j].memberRole !=
   // "RAMS
   // Reviewer")/*&&(returnedMembers[j].activeMember)*/){
   // skipFlag = true;
   // }
   // }
   // if(!skipFlag)
   // selectUsers.push(tableUsers[i].displayName);
   // }

   container = document.createElement("div");
   container.setAttribute("class", "form-content");

   /* Header Table */
   table = tableHeaderHelper("Personnel", [/* "VA", */
   "Name", "Study Team?", "Role", "COI Required?", "Edit Study?", "Purpose", "Notify?", "Update"
   ]);
   container.appendChild(table);

   /* Scrollable Body Table */
   tableDiv = document.createElement("div");
   tableDiv.setAttribute("class", "div-table-content-big");

   table = document.createElement("table");
   table.setAttribute("class", "table table-striped table-hover");

   /* Table Body */
   tbody = document.createElement("tbody");

   /* first row */
   // trow = document.createElement("tr");
   // //not va
   // trow.appendChild(checkboxCellHelper("initVA", "Is Person not in VA", "swapForNonVA('initVA', 'initPerson')"));
   // //blank space
   // trow.appendChild(textCellHelper("td", ""));
   // //trow.appendChild(selectCellHelper("initPerson", "Select Person to Add", ["Please Select a
   // User"].concat(selectUsers), null));
   // //study team
   // trow.appendChild(checkboxCellHelper("initStudyTeam", "Is Person on Study Team", null));
   // //blank space//role
   // trow.appendChild(textCellHelper("td", ""));
   // //trow.appendChild(selectCellHelper("initPersonRole", "Person's Role", ["Principal Investigator", "Clinical
   // Coordinator", "Contractor"], null));
   // //COI required
   // trow.appendChild(checkboxCellHelper("initCOIReq", "Does Person Require COI", null));
   // //write permissions
   // trow.appendChild(checkboxCellHelper("initWritePerm", "Does Person have Write Permissions", null));
   // //explination
   // trow.appendChild(textCellHelper("td", ""));
   // //trow.appendChild(textareaCellHelper("td", "initExplanation", "Explination of Person"));
   // //notifications
   // trow.appendChild(checkboxCellHelper("initNotify", "Does Person want notifications", null));
   // //add/remove
   // //trow.appendChild(buttonCellHelper("Add", "insertPerson()"));
   // trow.appendChild(textCellHelper("td", ""));
   // tbody.appendChild(trow);
   // /*Second row*/
   // trow = document.createElement("tr");
   // //blank
   // trow.appendChild(textCellHelper("td", ""));
   // //name
   // cell = selectCellHelper("initPerson", "Select Person to Add", ["Please Select a User"].concat(selectUsers),
   // null);
   // cell.setAttribute("colspan", "2");
   // trow.appendChild(cell);
   // //role
   // cell = selectCellHelper("initPersonRole", "Person's Role", ["Please Select a Role", "Clinical Coordinator",
   // "Contractor", "Scientist"], null)
   // cell.setAttribute("colspan", "2");
   // trow.appendChild(cell);
   // //purpose
   // cell = textareaCellHelper("td", "initExplanation", "Explination of Person");
   // cell.setAttribute("colspan", "3");
   // trow.appendChild(cell);
   // //add person
   // trow.appendChild(buttonCellHelper("Add", "insertPerson()"));
   // tbody.appendChild(trow);
   for (var i = 0; i < studyMembers.length; i++) {
      buttonID = "update" + i;
      buttonID_Remove = "remove" + i;

      trow = document.createElement("tr");
      trow.setAttribute("tabindex", "0");

      // non VA
      // trow.appendChild(returnChecked(returnedMembers[i].vaUser));
      // SPUserId - Hidden
      trow.appendChild(textCellHelper("td", studyMembers[i].user.SPUser, "SPU" + studyMembers[i].user.SPUser, "SPU"));
      // name
      var profileLink = "/sites/IRBApplications/_layouts/15/userdisp.aspx?ID=" + studyMembers[i].user.SPUser; 
      trow.appendChild(linkCellHelper("td", studyMembers[i].user.displayName, null, profileLink, "_blank"));
      // study team
      trow.appendChild(returnChecked(studyMembers[i].teamMember));
      // role
      trow.appendChild(textCellHelper("td", studyMembers[i].memberRole));
      // coi required
      trow.appendChild(returnChecked(studyMembers[i].coiRequired));
      // write permissions
      trow.appendChild(returnChecked(studyMembers[i].writePermissions));
      // explination
      trow.appendChild(textCellHelper("td", studyMembers[i].explaination));
      // notifications
      trow.appendChild(returnChecked(studyMembers[i].notifications));
      // remove
      // if(returnedMembers[i].activeMember == true){
      // trow.appendChild(buttonCellHelper("Deactivate", "deactivateUser('"+returnedMembers[i].user.usersID+"')"));
      // }else{
      // trow.appendChild(buttonCellHelper("Activate", "activateUser('"+returnedMembers[i].user.usersID+"')"));
      // }
      // console.log("member: "+ returnedMembers[i].user.displayName + " with role: " +
      // returnedMembers[i].memberRole);
      if ((studyMembers[i].memberRole == PRINCIPALINVESTIGATOR)
         || (studyMembers[i].memberRole == COPRINCIPALINVESTIGATOR) || (studyMembers[i].memberRole == PARENTSTUDYPI)
         || (studyMembers[i].memberRole == PARENTSTUDYCOPI) || (studyMembers[i].memberRole.indexOf("Reviewer") > -1)
         || (studyMembers[i].memberRole.indexOf(LOCALSITEINVESTIGATOR) > -1)
         || (studyMembers[i].memberRole.indexOf("Administrator") > -1)) {
         // disable update button and remove button for PI, Co-PI, Reviewers, and Administrators
         // the user needs to use the toolbar to change these entries
         trow.appendChild(buttonCellHelper("Update", "updatePerson('" + studyMembers[i].user.SPUser + "')", buttonID,
            true));
         trow.appendChild(buttonCellHelper("Remove", "removePerson('" + studyMembers[i].user.usersID + "')",
            buttonID_Remove, true));
      } else {
         // leave the update button and remove button enabled
         trow.appendChild(buttonCellHelper("Update", "updatePerson('" + studyMembers[i].user.SPUser + "')", buttonID));
         trow.appendChild(buttonCellHelper("Remove", "removePerson('" + studyMembers[i].user.usersID + "')",
            buttonID_Remove));
      }
      tbody.appendChild(trow);
   }

   table.appendChild(tbody);
   tableDiv.appendChild(table);
   // adding in the edit questions for the rows in the table
   // tableDiv.appendChild(editPersonnelDisplay(returnedMembers, tableUsers));
   container.appendChild(tableDiv);
   container.appendChild(editPersonnelDisplay(studyMembers, siteUsers));
   return container;
}

function buildUsernameList(JSON) {
   var tempList = [];
   for (var i = 0; i < JSON.d.results.length; i++) {
      tempList.push(JSON.d.results[i]);
   }
   return tempList;
}

function editPersonnelDisplay(studyMembers, siteUsers) {
   var container;
   var selectUsers = []; // contains final list of site users for drop down

   container = document.createElement("div");
   container.setAttribute("class", "form-content");

   var nameIndex, skipPerson;
   var studyMemberSPUsers = [];
   var siteUsersArray = buildUsernameList(siteUsers); // create array from siteUsers JSON data

   /* add all study member display names to list */
   for (var i = 0; i < studyMembers.length; i++) {
      studyMemberSPUsers.push(studyMembers[i].user.SPUser);
   }
   /* add site users to selectUsers array (based on role) */
   for (var j = 0; j < siteUsersArray.length; j++) {
      nameIndex = studyMemberSPUsers.indexOf(siteUsersArray[j].Id); // check if site user is on the study member
      // list
      if (nameIndex > -1) { // user is on the study member list
         skipPerson = false;
         var smRole = studyMembers[nameIndex].memberRole;
         if ((smRole.indexOf("Reviewer") > -1) || (smRole.indexOf("Administrator") > -1)
         // || (smRole == "Liaison Reviewer")
         || (smRole == PARENTSTUDYPI) || (smRole == PRINCIPALINVESTIGATOR) || (smRole == COPRINCIPALINVESTIGATOR)
            || (smRole.indexOf(LOCALSITEINVESTIGATOR) > -1)) {
            skipPerson = true;
         }
         if (!skipPerson) { // if study member is not a reviewer, PI or Co-PI
            selectUsers.push(siteUsersArray[j]);
         }
      } else { // user is not on the study member list
         selectUsers.push(siteUsersArray[j]);
      }
   }

   // need to capture how many pixels each row takes up, then we want to change the height of div-table-content-big
   // to reflect how much room the table actually

   // container.appendChild(radioFieldHelper("initVA", "Is the person you are adding in the VA?", ["Yes", "No"]));
   container.appendChild(selectFieldHelper("initPerson", "Select a user to add or update:", [ "Please Select a User"
   ].concat(selectUsers), "updatePerson()"));
   container.appendChild(radioFieldHelper("initStudyTeam", "Is this user on your study team?", [ "Yes", "No"
   ]));
   container.appendChild(selectFieldHelper("initPersonRole", "What role will this user play?", [ "Investigator",
      "Co-Investigator", "Sub-Investigator", "Biostatistician", "Clinical Coordinator", "Collaborator",
      "CSP Program Manager", "Data Manager", "Lab Manager", "Lab Technician", "National Study Coordinator",
      "Nurse Assessor", "Pharmacist", "Pharmacy Technician", "Regulatory Coordinator", "Research Assistant",
      "Research Nurse", "Statistician", "Study Coordinator"
   ]));
   container.appendChild(radioFieldHelper("initCOIReq", "Does this user have a Conflict of Interest?", [ "Yes", "No"
   ]));
   container.appendChild(radioFieldHelper("initWritePerm",
      "Do you want this user to be able to edit your application?", [ "Yes", "No"
      ]));
   container.appendChild(textFieldHelper("initExplanation", "If needed, please elaborate on the user being added."));
   container.appendChild(radioFieldHelper("initNotify",
      "Do you want this user to recieve notifications pertaining to this study?", [ "Yes", "No"
      ]));
   container.appendChild(radioFieldHelper("initActivate", "Do you want to give this user access to your application?",
      [ "Yes", "No"
      ]));

   return container;
}

function attachmentsDisplay(returnedFiles) {
   var container, link, paragraph, text;
   var table, tableDiv, tbody, trow, dateAdded;

   container = document.createElement("div");
   container.setAttribute("class", "form-content");

   /* Header Table */
   // if(readOnlyState == false){
   // table = tableHeaderHelper("Current Attachments", ["Attachment", "Date Added", "Version", "Status", "Date
   // Approved", "Document Type", "Remove"]);
   // }else{
   // table = tableHeaderHelper("Current Attachments", ["Attachment", "Date Added", "Version", "Status", "Date
   // Approved", "Document Type"]);
   // }
   /* Add link to template */
   paragraph = document.createElement("P");
   text = document.createTextNode("Click the link to download the ");

   link = document.createElement("A");
   link.href = "/sites/IRBApplications/FormTemplates/VHA%20form%2010-0493_Authorization%20for%20Use%20%20Release%20of%20IIHI%20for%20VHA%20Research%20(9-2015).pdf";
   link.target = "_blank"; // open document in new window or tab
   link.innerHTML = "HIPAA Template";

   paragraph.appendChild(text);
   paragraph.appendChild(link);
   container.appendChild(paragraph);

   table = tableHeaderHelper("Current Attachments", [ "Attachment", "Date Added", "Version", "Status", "Date Approved",
      "Document Type", "Remove", "Update"
   ]);

   container.appendChild(table);

   /* Scrollable Body Table */
   tableDiv = document.createElement("div");
   tableDiv.setAttribute("class", "div-table-content-big");

   table = document.createElement("table");
   table.setAttribute("class", "table table-striped table-hover");
   table.setAttribute("tabindex", "0");

   /* Table Body */
   tbody = document.createElement("tbody");

   for (var i = 0; i < returnedFiles.length; i++) {
      trow = document.createElement("tr");
      trow.setAttribute("tabindex", "0");

      trow.appendChild(linkCellHelper("td", (returnedFiles[i].Name), "Study%20Team", returnedFiles[i].Name));
      dateAdded = returnedFiles[i].TimeCreated.split("T")[1].replace("Z", "") + " on "
         + returnedFiles[i].TimeCreated.split("T")[0];
      trow.appendChild(textCellHelper("td", dateAdded));
      trow.appendChild(textCellHelper("td", returnedFiles[i].UIVersion)); // version //returnedFiles[i].UIVersion
      trow.appendChild(textCellHelper("td", returnedFiles[i].ListItemAllFields.Status)); // status
      trow.appendChild(textCellHelper("td", returnedFiles[i].ListItemAllFields.DateApproved)); // date approved
      trow.appendChild(textCellHelper("td", returnedFiles[i].ListItemAllFields.DocumentType)); // document type
      // if(readOnlyState == false){
      // trow.appendChild(buttonCellHelper("Remove", "removeAttachment('"+returnedFiles[i].Name+"',
      // 'Study%20Team')"));
      // }

      trow.appendChild(buttonCellHelper("Remove", "removeAttachment('" + returnedFiles[i].Name + "', 'Study%20Team')"));
      trow.appendChild(buttonCellHelper("Update", "updateCurrentAttachment('" + returnedFiles[i].ListItemAllFields.ID
         + "')"));

      tbody.appendChild(trow);
   }

   table.appendChild(tbody);
   tableDiv.appendChild(table);
   container.appendChild(tableDiv);
   return container;
}

function triggerUpdateModal() {
   var modal, modalDialog, modalContent, modalHeader;
   var header, heading1, lineBreak, updateButton, closeButton, text, modalFooter;
   // var container, tableHeader, table, tableDiv, tbody, trow, trow2, dateAdded, cell, modalTitle, modalBody;
   var statusTypes = [ "Approved", "Changes Requested", "Submitted", "Under Review"
   ];

   modal = document.createElement("div");
   modal.setAttribute("id", "updateModal");
   modal.setAttribute("class", "modal");

   modalDialog = document.createElement("div");
   modalDialog.setAttribute("role", "document");

   modalContent = document.createElement("div");
   modalContent.setAttribute("class", "modal-content");
   modalHeader = document.createElement("div");
   modalHeader.setAttribute("class", "modal-header");
   modalHeader.setAttribute("tabindex", "0"); // start tabbing on modal from header

   header = document.createElement("header");
   header.setAttribute("class", "modal-title");
   header.setAttribute("id", "modalTitle2");

   heading1 = document.createElement("H2");
   heading1.setAttribute("class", "modal-title");
   text = document.createTextNode("Update Document");
   heading1.appendChild(text);
   header.appendChild(heading1);
   modalHeader.appendChild(header);

   lineBreak = document.createElement("br");
   text = document.createTextNode("Please update the status of this document - " + currentAttachmentID
      + " - by selecting an option below.");

   modalFooter = document.createElement("div");
   modalFooter.setAttribute("class", "modal-footer");

   modalContent.appendChild(modalHeader);
   modalContent.appendChild(lineBreak);
   modalContent.appendChild(text);
   modalContent.appendChild(lineBreak);
   modalContent.appendChild(buildSelecterHelper("status", "Status", statusTypes, 12));
   modalContent.appendChild(lineBreak);

   /* Add Save Document Button */
   updateButton = document.createElement("button");
   updateButton.setAttribute("class", "btn btn-sm modal-button"); // took out btn-info
   updateButton.setAttribute("data-dismiss", "updateModal");
   updateButton.setAttribute("aria-label", "Update");

   updateButton.setAttribute("title", "Update your document"); // hover tip
   updateButton.setAttribute("type", "button");
   updateButton.setAttribute("role", "button");
   updateButton.setAttribute("id", "updateModal");

   updateButton.setAttribute("onclick", "updateAttachment()");
   text = document.createTextNode("Update Document");
   updateButton.appendChild(text);

   /* Add Close Window Button */
   closeButton = document.createElement("button");
   closeButton.setAttribute("class", "btn btn-sm modal-button modal-button-right");
   closeButton.setAttribute("data-dismiss", "modal");
   closeButton.setAttribute("aria-label", "Close");

   closeButton.setAttribute("title", "Close this window and return to the application"); // hover tip
   closeButton.setAttribute("type", "button");
   closeButton.setAttribute("role", "button");
   closeButton.setAttribute("id", "closeModal2");

   closeButton.setAttribute("onclick", "closeAttachmentUpdate()");
   text = document.createTextNode("Close this window");
   closeButton.appendChild(text);

   /* Combine Modal Elements */
   modalFooter.appendChild(updateButton);
   modalFooter.appendChild(closeButton);
   modalContent.appendChild(modalFooter);
   modalDialog.appendChild(modalContent);
   modal.appendChild(modalDialog);
   $('body').append(modal);
   $('#updateModal').popup('show');
}

/* exported updateCurrentAttachment */
function updateCurrentAttachment(currentID) {
   currentAttachmentID = currentID;
   triggerUpdateModal();
}
/* exported closeAttachmentUpdate */
function closeAttachmentUpdate() {
   $('#updateModal').popup('hide');
   refreshDisplay('Files');
}

/* exported enableInvestigatorDropdown */
/* enable local site investigator dropdown with users from the local site */
function enableInvestigatorDropdown() {
   var facility = $("#addLocalSite").find(':selected').val();
   var users = [];
   var option, text;

   if (facility !== "Please Select a Facility") {
      users = userObject[facility].users;

      // add available LSIs and enable dropdown
      $("#addLSI").children().remove(); // remove old options
      for (var i = 0; i < (users.length); i++) { // add new options
         if (typeof smcPlusIncumbents.smc[userObject[facility][users[i]].id] === "undefined") {
            option = document.createElement("option");
            option.setAttribute("value", userObject[facility][users[i]].id);

            text = document.createTextNode(users[i]);
            option.appendChild(text);
            $("#addLSI").append(option);
         }
      }
      $("#addLSI").removeAttr("disabled");
   }
}

// Add local site liaison to Reviewers group on application's list item
function addToReviewersGroup(json, localSiteLiaisonID, localSiteInvestigatorID) {
   var application = json.d.results;
   var listItemID = application[0].Id;
   var reviewers = application[0].ReviewersId;
   var newData;

   if ((typeof (reviewers) === "undefined") || ((reviewers) === null)) { // no users in reviewers group
      newData = {
         "__metadata" : {
            "type" : "SP.Data.ApplicationListItem"
         },
         "ReviewersId" : {
            'results' : [ localSiteLiaisonID, localSiteInvestigatorID
            ]
         }
      };
   } else { // at least one user in users group
      var reviewersArray = [];
      for (var i = 0; i < reviewers.results.length; i++) {
         reviewersArray.push(reviewers.results[i]);
      }
      reviewersArray.push(localSiteLiaisonID);
      reviewersArray.push(localSiteInvestigatorID);

      newData = {
         "__metadata" : {
            "type" : "SP.Data.ApplicationListItem"
         },
         "ReviewersId" : {
            'results' : reviewersArray
         }
      };
   }

   // Update Application list item
   $.ajax({
      url : siteUrl.concat("_api/Web/Lists/GetByTitle('Application')/Items(" + listItemID + ")"), // post change to
      // application list
      type : "POST",
      data : JSON.stringify(newData),
      headers : {
         "X-RequestDigest" : requestDigest,
         "Content-Type" : "application/json;odata=verbose",
         "IF-MATCH" : "*",
         "X-HTTP-Method" : "MERGE"
      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError((siteUrl.concat("_api/Web/Lists/GetByTitle('Application')/Items(" + listItemID + ")")), errorThrown,
            textStatus, JSON.stringify(XMLHttpRequest));
      }
   });
}

/* exported assignMultiSiteStudy */
/**
 * After the user confirms the new local site, add a row in SharePoint for the empty local site application. Add the
 * local site liaison for each site to the main application list item (to grant permissions)
 */
function assignMultiSiteStudy() {
   var localSite = $("#addLocalSite").find(':selected').val();
   var localSiteInvestigator = $("#addLSI").find(':selected').text();
   // var localSiteInvestigatorID = userObject[localSite][localSiteInvestigator].id;
   var localSiteInvestigatorID = $("#addLSI").find(':selected').val();
   var localSiteLiaisonID = userObject[localSite].localSiteLiaisonID;
   var localSiteLiaison = userObject[localSite].localSiteLiaison;

   // NOT USED principalInvestigatorID
   // var principalInvestigatorID = spAppData[RAMS.Environment.SharePoint.List.Application.localSiteInvestigatorID];
   var coPrincipalInvestigatorID = spAppData[RAMS.Environment.SharePoint.List.Application.CoPrincipalInvestigatorId];

   var userGroupID = userObject[localSite].userGroupID;
   var groupID = userObject[localSite].groupID;
   var sendData, data;

   if (consoleLoggingEnabled) {
      console.log("Local Site Selected: " + localSite + "\nLocal Site Investigator Selected: " + localSiteInvestigator
         + "\nLocal Site Investigator ID: " + localSiteInvestigatorID + "\nUser Group ID: " + userGroupID
         + "\nLocal Site ID: " + groupID + "\nLocal Site Liaison: " + localSiteLiaison + "\nLocal Site Liaison ID: "
         + localSiteLiaisonID);
   }

   var msg = "You have selected the following site and investigator:\n\nLocal Site Selected: " + localSite
      + "\nLocal Site Investigator Selected: " + localSiteInvestigator + "\nLocal Site Liaison: " + localSiteLiaison;
   if ((typeof smcPlusIncumbents.smc[localSiteInvestigatorID] !== "undefined")
      || (typeof smcPlusIncumbents.smc[localSiteLiaisonID] !== "undefined")) {
      sendData = alert(msg
         + "\n\nOne these people is already a member of the study team and they can't be added again."
         + "\n\nPress [OK] to return.");
      sendData = false;
   } else {
      sendData = confirm(msg + "\n\nPress [OK] to confirm your choices.");
   }
   if (sendData) {
      // When adding the Local Site Application shell, add the Local Site Investigator and Local Site Liaison to the
      // Parent study application
      insertStudyMember(localSiteLiaisonID, localSite + " Liaison Reviewer", false, assignedID, null);
      insertStudyMember(localSiteInvestigatorID, localSite + " Local Site Investigator", false, assignedID, null);

      // 1) Add list item for local site application (61) to Application list
      data = {
         "__metadata" : {
            "type" : "SP.Data.ApplicationListItem"
         },
         "ParentStudyID" : assignedID, // check
         "ParticipatingFacilitiesLookupId" : groupID, // TODO TAR Setting ParticipatingSiteId needs to use Facilities
         // Ids
         "ParticipatingSiteInvestigatorId" : localSiteInvestigatorID,
         "ParticipatingSiteLiaisonId" : localSiteLiaisonID,
         "ParticipatingSiteApplicationStat" : "Not Started",
         "PrincipalInvestigatorId" : localSiteInvestigatorID,
         "CoPIId" : coPrincipalInvestigatorID
      };

      $.ajax({
         type : 'POST',
         data : JSON.stringify(data),
         url : siteUrl.concat("_api/web/lists/getbytitle('Application')/items"),
         headers : {
            "X-RequestDigest" : requestDigest,
            "content-type" : "application/json;odata=verbose"
         },
         error : function(XMLHttpRequest, textStatus, errorThrown) {
            logError(restUrl, errorThrown, textStatus, JSON.stringify(XMLHttpRequest));
         }
      });

      // 2) Add local site liaison to reviewers group on main application (60) list item
      $.ajax({
         type : "GET",
         url : siteUrl.concat("_api/Web/Lists/GetByTitle('Application')/Items" + "?$filter=Study_ID eq '" + assignedID
            + "'"),
         headers : {
            "accept" : "application/json; odata=verbose"
         },
         dataType : 'json',
         success : function(json) {
            addToReviewersGroup(json, localSiteLiaisonID, localSiteInvestigatorID);
         },
         error : function(XMLHttpRequest, textStatus, errorThrown) {
            logError((siteUrl.concat("_api/Web/Lists/GetByTitle('Application')/Items" + "?$filter=Study_ID eq '"
               + assignedID + "'")), errorThrown, textStatus, JSON.stringify(XMLHttpRequest));
         }
      });
      // 662284: When Creating A Local Site Application The Facility Name Is Not Removed From The Dropdown
      $("#addLocalSite option[value='" + localSite + "']").remove();
   }
}

/* exported closeAddSiteModal */
/**
 * turn off the modal window and remove it and all addSiteModal ids, close window for adding local sites, and refresh
 * the page
 */
function closeAddSiteModal() {
   $('#addSiteModal').popup('toggle'); // or can use .popup('hide');
   // this searches for all dom elements with an id that begins with addSiteModal. There whould be 3 found,
   // addSiteModal, addSiteModal_wrapper, and addSiteModal_background
   $('[ id ^= addSiteModal ]').remove();
   refreshDisplay('Facility');
}

/* exported triggerAddSiteModal */
/**
 * open window that allows user to add local site (w/ local site investigator) to their study
 */
function triggerAddSiteModal() {
   var modal, modalDialog, modalContent, modalHeader;
   var header, heading1, lineBreak, saveButton, closeButton;
   var paragraph, text, modalFooter;
   var container, textDiv, table, tableDiv, tbody, trow, trow2;

   var selectFacilities = [];
   var localSites = [];

   // create list of selected local sites
   for ( var site in localSiteAppObject) {
      if (!localSiteAppObject.hasOwnProperty(site)) {
         continue;
      }
      localSites.push(localSiteAppObject[site].siteName);
   }

   // NOT USED facilityId
   // var facilityId = spAppData.FacilitiesLookupId;
   var facilityName = getFacilityNameById(spAppData.FacilitiesLookupId);

   // create list of available facilities
   for ( var key in userObject) {
      if (!userObject.hasOwnProperty(key)) {
         continue;
      }

      if ((localSites.indexOf(key) < 0) && (key != "Central IRB") && (key != facilityName)) {
         selectFacilities.push(key);
      }
   }
   selectFacilities.sort();

   modal = document.createElement("div");
   modal.setAttribute("class", "modal");
   modal.setAttribute("id", "addSiteModal");
   modal.setAttribute("tabindex", "-1");
   modal.setAttribute("role", "dialog");
   modal.setAttribute("aria-labelledby", "addSiteModal");

   modalDialog = document.createElement("div");
   modalDialog.setAttribute("role", "document");

   modalContent = document.createElement("div");
   modalContent.setAttribute("class", "modal-content");
   modalHeader = document.createElement("div");
   modalHeader.setAttribute("class", "modal-header");
   modalHeader.setAttribute("tabindex", "0"); // start tabbing on modal from header

   header = document.createElement("header");
   header.setAttribute("class", "modal-title");
   header.setAttribute("id", "modalTitle");

   heading1 = document.createElement("H2");
   heading1.setAttribute("class", "modal-title");
   text = document.createTextNode("Add Local Site");
   heading1.appendChild(text);
   header.appendChild(heading1);
   modalHeader.appendChild(header);

   textDiv = document.createElement("div");
   textDiv.setAttribute("class", "col-xs-12");
   paragraph = document.createElement("p");
   lineBreak = document.createElement("br");
   text = document
      .createTextNode("To successfully add a participating site to your application, select the facility and the site investigator from the respective dropdown lists. Then click the [Save Data] button to record your changes. Click the [Close This Window] button to return to the table.");

   paragraph.appendChild(lineBreak);
   paragraph.appendChild(text);
   textDiv.appendChild(paragraph);
   modalContent.appendChild(textDiv);

   modalContent.appendChild(modalHeader);
   modalContent.appendChild(lineBreak);
   modalContent.appendChild(text);
   modalContent.appendChild(lineBreak);

   /* Setup for Adding Local Site */
   container = document.createElement("div");
   container.setAttribute("class", "form-content");

   /* Table Division */
   tableDiv = document.createElement("div");
   tableDiv.setAttribute("class", "div-table-content-big"); // div-table-content-big should be small?

   /* Table Style */
   table = document.createElement("table");
   table.setAttribute("class", "table table-hover");
   table.setAttribute("tabindex", "0");

   /* Table Body & Rows */
   tbody = document.createElement("tbody");
   trow = document.createElement("tr");
   trow.setAttribute("tabindex", "0");
   trow2 = document.createElement("tr");

   // trow.appendChild(selectCellHelper("addLocalSite", "Select Facility to add", ["Please Select a
   // Facility"].concat(selectFacilities), "enableInvestigatorDropdown()"));
   // trow2.appendChild(selectCellHelper("addLSI", "Select Investigator", ["Please Select an Investigator"], null,
   // true));

   trow.appendChild(buildSelecterHelper("addLocalSite", "Selected Facility: ", [ "Please Select a Facility"
   ].concat(selectFacilities), 12, "enableInvestigatorDropdown()"));
   trow2.appendChild(buildSelecterHelper("addLSI", "Selected Local Site Investigator: ",
      [ "Please Select an Investigator"
      ], 12));

   /* Add Table Elements Together */
   tbody.appendChild(trow);
   tbody.appendChild(trow2);
   table.appendChild(tbody);
   tableDiv.appendChild(table);

   container.appendChild(lineBreak);
   container.appendChild(tableDiv);

   modalFooter = document.createElement("div");
   modalFooter.setAttribute("class", "modal-footer");

   /* Add Save Button */
   saveButton = document.createElement("button");
   saveButton.setAttribute("class", "btn btn-sm modal-button");
   saveButton.setAttribute("aria-label", "Save Data");

   saveButton.setAttribute("title", "Save your site and investigator selections"); // hover tip
   saveButton.setAttribute("type", "button");
   saveButton.setAttribute("role", "button");
   saveButton.setAttribute("id", "saveLocalSite");

   saveButton.setAttribute("onclick", "assignMultiSiteStudy()");
   text = document.createTextNode("Save Data");
   saveButton.appendChild(text);

   /* Add Close Window Button */
   closeButton = document.createElement("button");
   closeButton.setAttribute("class", "btn btn-sm modal-button modal-button-right");
   closeButton.setAttribute("data-dismiss", "modal");
   closeButton.setAttribute("aria-label", "Close");

   closeButton.setAttribute("title", "Close this window and return to the table"); // hover tip
   closeButton.setAttribute("type", "button");
   closeButton.setAttribute("role", "button");
   closeButton.setAttribute("id", "closeModal");

   closeButton.setAttribute("onclick", "closeAddSiteModal()");
   text = document.createTextNode("Close This Window");
   closeButton.appendChild(text);

   /* Combine Modal Elements */
   modalContent.appendChild(container);
   modalFooter.appendChild(saveButton);
   modalFooter.appendChild(closeButton);
   modalContent.appendChild(modalFooter);
   modalDialog.appendChild(modalContent);
   modal.appendChild(modalDialog);
   $('body').append(modal);

   $('#addSiteModal').popup('show');
}

/* add the user title to the global object localSiteAppObject */
function addUser(data, siteID, property) {
   var title = data.d.Title;
   localSiteAppObject[siteID][property] = title;
}

/* return the JSON of the user */
function getUserById(userID, siteID, property) {
   return $.ajax({
      async : false,
      type : 'GET',
      url : siteUrl.concat("_api/Web/GetUserById(" + userID + ")/"),
      headers : {
         "accept" : "application/json;odata=verbose"
      },
      dataType : 'json',
      success : function(json) {
         addUser(json, siteID, property);
      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError((siteUrl.concat("_api/Web/GetUserById(" + userID + ")/")), errorThrown, textStatus, JSON
            .stringify(XMLHttpRequest));
      }
   });
}

// /**
// * add the facility title to the global object localSiteAppObject
// *
// * @param {json} data the facility record from the get _api/Web/Lists/GetByTitle('Facility')/GetItemById({siteID})
// call
// * @param {number} siteID the number of the facility in the Facility List
// * @param {string} property the name of the attribute to set the facility title into in the localSiteAppObject
// */
// function addFacilityName(data, siteID, property) {
// var title = data.d.FacilityName;
// localSiteAppObject[siteID][property] = title;
// }

/**
 * Read the Facility List by the siteID and set the facilityName into the property of the localSiteAppObject. This is a
 * synchronous call
 * 
 * @param {number} siteID the number of the facility in the Facility List
 * @param {string} property the name of the attribute to set the facility title into in the localSiteAppObject
 */
function getFacilityName(siteID, property) {
   if (siteID) {
      return $.ajax({
         async : false,
         type : 'GET',
         url : siteUrl.concat("_api/Web/Lists/GetByTitle('Facilities')/GetItemById(" + siteID + ")/"),
         headers : {
            "accept" : "application/json;odata=verbose"
         },
         dataType : 'json',
         success : function(json) {
            // addFacilityName(json, siteID, property);
            localSiteAppObject[siteID][property] = json.d.Title;
         },
         error : function(XMLHttpRequest, textStatus, errorThrown) {
            logError((siteUrl.concat("_api/Web/Lists/GetByTitle('Facilities')/GetItemById(" + siteID + ")/")),
               errorThrown, textStatus, JSON.stringify(XMLHttpRequest));
         }
      });
   } else {
      if (consoleLoggingEnabled) {
         console.log("ERROR: getFacilityName called with invalid siteID: " + siteID);
      }
      // If this alert is seen by the user it is most likely a data issue or data construction issue.
      alert("Invalid site. Please contact support.");
   }
}

/*
 * Loop through local site SharePoint applications and add the data to the localSiteAppObject for later reference
 */
function buildLocalSiteObject(json) {
   var applications = json.d.results;
   localSiteAppObject = {};
   var parentStudyID, appStatus, siteID, investigatorID, liaisonID, hyperlink, studyID, studyTitle;

   for (var i = 0; i < applications.length; i++) {
      parentStudyID = applications[i].ParentStudyID;
      appStatus = applications[i].ParticipatingSiteApplicationStat;
      if (applications[i].ParticipatingFacilitiesLookupId) {
         siteID = applications[i].ParticipatingFacilitiesLookupId;
      } else {
         siteID = applications[i].FacilitiesLookupId;
      }

      investigatorID = applications[i].ParticipatingSiteInvestigatorId;

      liaisonID = applications[i].ParticipatingSiteLiaisonId;
      hyperlink = applications[i].LinktoApplication;
      studyID = applications[i].Study_ID;
      studyTitle = applications[i].StudyTitle;

      localSiteAppObject[siteID] = {};
      localSiteAppObject[siteID].appStatus = appStatus;
      localSiteAppObject[siteID].siteID = siteID;
      localSiteAppObject[siteID].investigatorID = investigatorID;

      localSiteAppObject[siteID].liaisonID = liaisonID;
      localSiteAppObject[siteID].linkToApplication = hyperlink;
      localSiteAppObject[siteID].studyID = studyID;
      localSiteAppObject[siteID].studyTitle = studyTitle;

      console.log("Parent Study ID: " + parentStudyID + "\nLocal Site Application Status: " + appStatus
         + "\nLocal Site ID: " + siteID + "\nLocal Site Investigator ID: " + investigatorID
         + "\nLocal Site Liaison ID: " + liaisonID);
      getUserById(investigatorID, siteID, "investigator");
      getUserById(liaisonID, siteID, "liaison");
      getFacilityName(siteID, "siteName");
   }
}

/*
 * Grab all local site SharePoint applications and return the json data for each list item
 */
function grabLocalSiteApplications(assignedID) {
   var applicationUrl = siteUrl.concat("_api/web/lists/getbytitle('Application')/items?$filter=ParentStudyID eq '"
      + assignedID + "'");
   return $.ajax({
      async : false,
      type : 'GET',
      url : applicationUrl,
      headers : {
         "accept" : "application/json;odata=verbose"
      },
      dataType : 'json',
      success : function(json) {
         buildLocalSiteObject(json);
      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError((siteUrl.concat("_api/Web/Lists/GetByTitle('Application')/items?$filter=ParentStudyID eq '"
            + assignedID + "')/")), errorThrown, textStatus, JSON.stringify(XMLHttpRequest));
      }
   });
}

/* exported invokeChildStudy */
/* invoke function call to create child study/local site application */
// Passing in Local Site Investigator to be inserted as study member for edit permission
function invokeChildStudy(siteID, investigatorID) {
   generateAssignedId("Local", siteID, investigatorID);
}

/*
 * generate local site application page: page allows user to start a new local site study and designate both local site
 * investigator (LSI) and local site liaison (LSL)
 */
function facilityDisplay() {
   var container, table, tableDiv, tbody, trow, counter, buttonID;
   var selectFacilities = [];
   var disable = true;

   // create list of facilities
   for ( var key in userObject) {
      if (!userObject.hasOwnProperty(key)) {
         continue;
      }
      selectFacilities.push(key);
   }

   container = document.createElement("div");
   container.setAttribute("class", "form-content");

   /* Header Table */
   table = tableHeaderHelper("Local Site Applications", [ "Add Site", "Site Name", "Local Site Investigator",
      "Local Site Liaison", "Application Name", "Application Link"
   ]);
   container.appendChild(table);

   /* Scrollable Body Table */
   tableDiv = document.createElement("div");
   tableDiv.setAttribute("class", "div-table-content-big");

   table = document.createElement("table");
   table.setAttribute("class", "table table-striped table-hover");
   table.setAttribute("tabindex", "0");

   /* Table Body */
   tbody = document.createElement("tbody");

   // add row for each local site application
   counter = 0;
   for ( var site in localSiteAppObject) {
      if (!localSiteAppObject.hasOwnProperty(site)) {
         continue;
      }
      trow = document.createElement("tr"); // create clean row
      trow.appendChild(textCellHelper("td", "Site Added"));
      // use ID to get facility
      trow.appendChild(textCellHelper("td", (localSiteAppObject[site].siteName)));
      trow.appendChild(textCellHelper("td", (localSiteAppObject[site].investigator)));
      trow.appendChild(textCellHelper("td", (localSiteAppObject[site].liaison)));

      /*
       * Allow access to created local site study 61 if already initiated. Allow ability to create local site study 61
       * based on status of main study 60.
       */
      if ((localSiteAppObject[site].appStatus == RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFTCHOICE) && (localSiteAppObject[site].linkToApplication)) {
         // study initiated, show study title and link
         trow.appendChild(textCellHelper("td", localSiteAppObject[site].studyTitle));
         trow.appendChild(linkCellHelper("td", localSiteAppObject[site].studyID, null,
            localSiteAppObject[site].linkToApplication.Url, "_blank"));
      } else if ((spAppData.ApplicationStatus == RAMS.Environment.SharePoint.List.CommitteeStatus.APPROVEDCHOICE)
         || (spAppData.ApplicationStatus == RAMS.Environment.SharePoint.List.CommitteeStatus.APPROVEDWITHMAJORMODIFICATIONSCHOICE)
         || (spAppData.ApplicationStatus == RAMS.Environment.SharePoint.List.CommitteeStatus.APPROVEDWITHMINORMODIFICATIONSCHOICE)) {
         // main study approved, enable start application button
         trow.appendChild(textCellHelper("td", "No Application Name Available"));
         buttonID = "startapplication" + localSiteAppObject[site].siteID;
         if ((currentUser.Id == localSiteAppObject[site].investigatorID)
            || (currentUser.Id == localSiteAppObject[site].liaisonID)) {
            disable = false;
         } else {
            disable = true;
         }
         trow.appendChild(buttonCellHelper("Start Application", ('invokeChildStudy(' + localSiteAppObject[site].siteID
            + ',' + localSiteAppObject[site].liaisonID + ')'), buttonID, disable));
      } else { // main study not approved, disable start application button
         trow.appendChild(textCellHelper("td", "No Application Name Available"));
         buttonID = "startapplication" + localSiteAppObject[site].siteID;
         trow.appendChild(buttonCellHelper("Start Application", ('invokeChildStudy(' + localSiteAppObject[site].siteID
            + ',' + localSiteAppObject[site].liaisonID + ')'), buttonID, true));
      }

      tbody.appendChild(trow);
      counter++;
   }

   // create row for adding local sites
   trow = document.createElement("tr");
   trow.appendChild(buttonCellHelper("Add", "triggerAddSiteModal()"));
   trow.appendChild(textCellHelper("td", "No Facility Selected"));
   trow.appendChild(textCellHelper("td", "No Investigator Selected"));
   trow.appendChild(textCellHelper("td", "No Liaison Selected"));
   trow.appendChild(textCellHelper("td", "No Application Name Available"));
   trow.appendChild(textCellHelper("td", "No Application Link Available"));
   tbody.appendChild(trow);

   table.appendChild(tbody);
   tableDiv.appendChild(table);
   container.appendChild(tableDiv);
   return container;
}

/* exported generateLocalApp */
function generateLocalApp(facilityID) {
   // console.log("generating local application");
   generateAssignedId("Local", facilityID);
}

/* exported swapForNonVA */
function swapForNonVA(checkboxID, swapID) {
   var tempList;
   var selectUsers = [];

   // remove drop down
   // $('#'+swapID).parent().children().remove();

   if ($('#' + checkboxID).is(':checked')) {
      $('#' + swapID).parent().replaceWith(textareaCellHelper("td", swapID, "Type in Non VA Person"));
   } else {
      tempList = ramsUsers.values();

      for (var i = 0; i < tempList.length; i++) {
         selectUsers.push(tempList[i].Title);
      }

      $('#' + swapID).parent().replaceWith(selectCellHelper(swapID, "Select Person to Add", selectUsers, null));
   }
}

/* exported insertFacility */
function insertFacility(id) {
   var facility = $('#' + id).val();
   // var facilities = spAppData.CoordinatingSitesId.results;
   // var liaisons;
   var data, liaisons, facilities;

   if (facility != "Please Select a Facility") {
      sendREST("GET", siteUrl.concat("_api/web/lists/getbytitle('Facilities')/items"), null, function(json) {
         var returnedFacilities = json.d.results;

         if ((typeof spAppData.CoordinatingSitesLiaisonId === "undefined")
            || (spAppData.CoordinatingSitesLiaisonId === null)) {
            liaisons = [];
         } else {
            liaisons = spAppData.CoordinatingSitesLiaisonId.results;
         }

         if (spAppData.CoordinatingSitesId.results.length === 0) {
            facilities = [];
         } else {
            // console.log("not null " + JSON.stringify(spAppData.CoordinatingSitesId));
            facilities = spAppData.CoordinatingSitesId.results;
         }

         for (var i = 0; i < returnedFacilities.length; i++) {
            if (returnedFacilities[i].Title == facility) {
               facilities.push(returnedFacilities[i].Id);
               liaisons.push(returnedFacilities[i].FacilityLiaisonId);

               // console.log("we are sending: " + liaisons + " and " + facilities);

               data = {
                  "__metadata" : spAppData.__metadata,
                  "CoordinatingSitesId" : {
                     "__metadata" : {
                        "type" : "Collection(Edm.Int32)"
                     },
                     "results" : facilities
                  },
                  "CoordinatingSitesLiaisonId" : {
                     "__metadata" : {
                        "type" : "Collection(Edm.Int32)"
                     },
                     "results" : liaisons
                  }
               };

               spAppData.CoordinatingSitesId.results = facilities;
               spAppData.CoordinatingSitesLiaisonId = {
                  "results" : liaisons
               };

               sendREST("POST", spAppData.__metadata.uri, data, function() {
                  // TAR No need to update subsite app list - it is not used anymore
                  // if (spsubsiteAppData != null) {
                  // sendREST("POST", spsubsiteAppData.__metadata.uri, data, function() {
                  // refreshDisplay("Facility");
                  // }, "Merge");
                  // } else {
                  refreshDisplay("Facility");
                  // }
               }, "Merge");
            }
         }
      });
   }
}

/* exported removeFacility */
function removeFacility(facilityID, liaisonID) {
   var facilities = [];
   var liaisons = [];
   var data;
   // console.log("we recieved " + facilityID + " and " + liaisonID);

   // grab facilities from appData, identify which one you don't need and resend merge
   // sendREST("GET", siteUrl.concat("RAMSIRBSC/_api/web/lists/getbytitle('Facility')/items"), null, function(json){
   // var returnedFacilities = json.d.results;
   // for(var i=0; i<returnedFacilities.length; i++){
   // if(returnedFacilities[i].Id == id){
   for (var j = 0; j < spAppData.CoordinatingSitesId.results.length; j++) {
      if (spAppData.CoordinatingSitesId.results[j] != facilityID) {
         facilities.push(spAppData.CoordinatingSitesId.results[j]);
      }
   }
   for (j = 0; j < spAppData.CoordinatingSitesLiaisonId.results.length; j++) {
      // console.log("checking " + spAppData.CoordinatingSitesLiaisonId.results[j] + " vs " + liaisonID);
      if (spAppData.CoordinatingSitesLiaisonId.results[j] != liaisonID) {
         // console.log("we want to keep: " + spAppData.CoordinatingSitesLiaisonId.results[j]);
         liaisons.push(spAppData.CoordinatingSitesLiaisonId.results[j]);
      }
   }

   if (liaisons.length >= 1) {
      data = {
         "__metadata" : spAppData.__metadata,
         "CoordinatingSitesId" : {
            "__metadata" : {
               "type" : "Collection(Edm.Int32)"
            },
            "results" : facilities
         },
         "CoordinatingSitesLiaisonId" : {
            "__metadata" : {
               "type" : "Collection(Edm.Int32)"
            },
            "results" : liaisons
         }
      };

      spAppData.CoordinatingSitesLiaisonId = {
         "results" : liaisons
      };
   } else {
      // console.log("packaging empty data");
      data = {
         "__metadata" : spAppData.__metadata,
         "CoordinatingSitesId" : {
            "__metadata" : {
               "type" : "Collection(Edm.Int32)"
            },
            "results" : facilities
         },
         "CoordinatingSitesLiaisonId" : {
            "__metadata" : {
               "type" : "Collection(Edm.Int32)"
            },
            "results" : liaisons
         }
      };
      spAppData.CoordinatingSitesLiaisonId = {
         "results" : liaisons
      };
   }

   spAppData.CoordinatingSitesId.results = facilities;

   // console.log("we are sending liaisons: " + liaisons + ", facilities: " + facilities);

   sendREST("POST", spAppData.__metadata.uri, data, function() {
      // TAR No need to update subsite app list anymore - it is not used.
      // if (spsubsiteAppData != null) {
      // sendREST("POST", spsubsiteAppData.__metadata.uri, data, function() {
      // refreshDisplay("Facility");
      // }, "Merge");
      // } else {
      refreshDisplay("Facility");
      // }
   }, "Merge");
   // }
   // }
   // });
}

/* exported insertPerson */
function insertPerson() {
   // Start the Manage Personnel Workflow
   // BEGIN FROM Sustainment 2.1.2.1
   // startRAMSWorkflow("D9C79A8E-CD98-4176-9D11-9C8E91D62CFF", null);
   // END FROM Sustainment 2.1.2.1

   var cirbUsersGroups;
   var usersGroups;
   var allUsersGroups = "";

   if (spAppData.ApplicationStatus != RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFTCHOICE) {
      var facilityName = getFacilityNameById(spAppData[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId]);
      ficObj.get(RAMS.Environment.SharePoint.CentralIRBSite, function(cirbFIObj) {
         ficObj.get(facilityName, function(fiObj) {
            if (spAppData.ApplicationSiteType == "Central") {
               cirbUsersGroups = cirbFIObj.getCommitteeGroup(IRB).getCoordinators().getName();
            }
            var revRequired = spAppData.ReviewRequired.split(",");
            if ((typeof revRequired !== "undefined") && (revRequired !== null) && (revRequired !== "")) {
               for ( var c in revRequired) {
                  usersGroups = (typeof usersGroups === "undefined" ? "" : usersGroups + ";")
                     + fiObj.getCommitteeGroup(revRequired[c]).getCoordinators().getName();
               }
            } else {
               usersGroups = "";
            }
            allUsersGroups = (typeof cirbUsersGroups === "undefined" ? "" : cirbUsersGroups + ";") + usersGroups;

            // manage personnel notification workflow
            // TemplateID = D9C79A8E-CD98-4176-9D11-9C8E91D62CFF
            // Internal SubscriptionID = D2E9118A-1236-4FB4-A3A1-08F857249574
            // Use the TemplateID
            //
            // The manage personnel notification workflow takes the Study_ID, To, Subject, and Body parameters.
            // It Logs the To Parameter to Workflow History, builds a study link to get to the study, builds a variable
            // to hold study details, then builds an email message and sends it.
            startRAMSWorkflow("D9C79A8E-CD98-4176-9D11-9C8E91D62CFF", null, allUsersGroups);
         });
      });
   }

   var userToMatch = $('#initPerson').val();
   if (userToMatch != "Please Select a User") {
      // check to see if person is in the sql table
      // if($('#initVA').is(':checked')){
      // //person isn't in VA, needs to be added
      // }else{
      sendREST(
         "GET",
         restPath.concat("study/" + assignedID + "/members"),
         null,
         function(returnedMembers) {
            var foundUser = null;

            for (var j = 0; j < returnedMembers.length; j++) {
               if (returnedMembers[j].user.SPUser == userToMatch) {
                  foundUser = returnedMembers[j];
               }
            }

            if (!foundUser) {
               sendREST(
                  "GET",
                  restPath.concat("user"),
                  null,
                  function(returnedUsers) {
                     for (var i = 0; i < returnedUsers.length; i++) {
                        // console.log("comparing: " + returnedUsers[i].displayName + " to " +
                        // $('#initPerson').val());
                        if (returnedUsers[i].SPUser == $('#initPerson').val()) {
                           foundUser = returnedUsers[i];
                           sendPersonnelTable(returnedUsers[i]);
                           break;
                        }
                     }
                     // TODO TAR If the person is not found in the RAMS DB Users issue an error message
                     if (!foundUser) {
                        insertWarning(
                           "The selected user is not in the RAMS database. They need to register by stating a Study Application before you can proceed.",
                           "alert-warning");
                     }
                  });
            } else {
               sendPersonnelTable(foundUser.user, foundUser.study_MembersID);
            }
         });

      // }
   } else {
      insertWarning("You must first select a User before you can proceed.", "alert-warning");
   }
}

function sendPersonnelTable(foundUser, memberID) {
   var data = {
      "study_MembersID" : memberID,
      "user" : foundUser,
      "vaUser" : $('#initVA_0').is(':checked'),
      "teamMember" : $('#initStudyTeam_0').is(':checked'),
      "memberRole" : $('#initPersonRole').val(),
      "coiRequired" : $('#initCOIReq_0').is(':checked'),
      "writePermissions" : $('#initWritePerm_0').is(':checked'),
      "explaination" : $('#initExplanation').val(),
      "notifications" : $('#initNotify_0').is(':checked'),
      "activeMember" : $('#initActivate_0').is(':checked')
   };
   // console.log("trying to send: " + JSON.stringify(data));
   // console.log("we are trying to add " + JSON.stringify(foundUser) + " with " + JSON.stringify(data));
   // sendREST("POST", restPath.concat("study/"+assignedID+"/members/"+foundUser.usersID), data, function(){
   // BEGIN FROM Sustainment Release 2.1.2.1
   var tempStudyID = spAppData.Study_ID;
   sendREST("GET", siteUrl.concat("_api/web/lists/getbytitle('Application')/items?$filter= Study%5FID eq '"
      + tempStudyID + "'"), null, function(json) {
      var returnedItem = json.d.results;
      var notifiers;
      if (returnedItem.length > 0) { // Do not want null reference
         if ((typeof returnedItem[0].NotifiersId === "undefined") || (returnedItem[0].NotifiersId === null)) {
            notifiers = [];
         } else {
            notifiers = returnedItem[0].NotifiersId.results;
         }
      }
      notifiers.push(foundUser.SPUser);
      var newData = {
         "__metadata" : spAppData.__metadata, // {"type": "SP.Data.ApplicationListItem"},
         "NotifiersId" : {
            "__metadata" : {
               "type" : "Collection(Edm.Int32)"
            },
            "results" : notifiers
         // WRONG!! Has to be SP User ID. Have to create a new function to map to SP
         // UserID from Members
         }
      };

      sendREST("POST", spAppData.__metadata.uri, newData, function() {
         console.log("the added member's id is " + foundUser.SPUser);
      }, "Merge");
   });

   // END FROM Sustainment Release 2.1.2.1

   var AOMember = aoMemberCheck();

   // TODO TAR There is no need for aomembers and members service calls to be seperate the functions do the same
   // thing.
   // compare the methods in detail and remove the aomembers service.
   if (AOMember === true) {
      sendREST("POST", restPath.concat("study/" + assignedID + "/aomembers/"), data, function() {
         refreshDisplay('Personnel');
      });

   } else {
      sendREST("POST", restPath.concat("study/" + assignedID + "/members/"), data, function() {
         refreshDisplay('Personnel');
      });
   }
}

/* exported removePerson */
function removePerson(removeUserID) {
   sendREST("GET", restPath.concat("study/" + assignedID + "/members"), null, function(returnedMembers) {
      var foundDelUser = null;

      for (var j = 0; j < returnedMembers.length; j++) {
         if (returnedMembers[j].user.usersID == removeUserID) {
            foundDelUser = returnedMembers[j];
         }
      }

      var data = {
         "study_MembersID" : foundDelUser.study_MembersID,
         "user" : foundDelUser.user,
         "teamMember" : foundDelUser.teamMember,
         "memberRole" : foundDelUser.memberRole,
         "coiRequired" : foundDelUser.coiRequired,
         "writePermissions" : foundDelUser.writePermissions,
         "explaination" : foundDelUser.explaination,
         "notifications" : foundDelUser.notifications,
         "activeMember" : foundDelUser.activeMember

      };

      sendREST("POST", restPath.concat("study/" + assignedID + "/removemembers/"), data, function() {
         refreshDisplay('Personnel');
      });
   });
}

/* exported updatePerson */
function updatePerson(updateUser) {
   var updatePerson = updateUser;
   if ((updatePerson === null) || (typeof updatePerson === "undefined")) {
      updatePerson = $('#initPerson').val();
   }
   // TODO TAR Need to reset form or clear prior entries before populating or if not found.
   sendREST("GET", restPath.concat("study/" + assignedID + "/members"), null, function(returnedMembers) {
      for (var i = 0; i < returnedMembers.length; i++) {
         if (returnedMembers[i].user.SPUser == updatePerson) {
            // console.log("found our user: " + returnedMembers[i].user.displayName);

            // name
            $('#initPerson').val(returnedMembers[i].user.SPUser);
            $('#initPerson').prop("disabled", true);

            // on study team
            if (returnedMembers[i].teamMember) {
               $('#initStudyTeam_0').attr('checked', 'checked');
            } else {
               $('#initStudyTeam_1').attr('checked', 'checked');
            }

            // role
            $('#initPersonRole').val(returnedMembers[i].memberRole);

            // COI
            if (returnedMembers[i].coiRequired) {
               $('#initCOIReq_0').attr('checked', 'checked');
            } else {
               $('#initCOIReq_1').attr('checked', 'checked');
            }

            // read/write
            if (returnedMembers[i].writePermissions) {
               $('#initWritePerm_0').attr('checked', 'checked');
            } else {
               $('#initWritePerm_1').attr('checked', 'checked');
            }

            // user explination
            $('#initExplanation').val(returnedMembers[i].explaination);

            // notifications
            if (returnedMembers[i].notifications) {
               $('#initNotify_0').attr('checked', 'checked');
            } else {
               $('#initNotify_1').attr('checked', 'checked');
            }

            // activate
            if (returnedMembers[i].activeMember) {
               $('#initActivate_0').attr('checked', 'checked');
            } else {
               $('#initActivate_1').attr('checked', 'checked');
            }
         }
      }
   });
}

function returnChecked(boolean) {
   if (boolean) {
      return textCellHelper("td", "x");
   }

   return textCellHelper("td", "");
}

/* exported deactivateUser */
function deactivateUser(usersID) {
   sendREST("POST", restPath.concat("study/" + assignedID + "/members/" + usersID + "/deactivate"), null, function() {
      refreshDisplay("Personnel");
   });
}

/* exported activateUser */
function activateUser(usersID) {
   sendREST("POST", restPath.concat("study/" + assignedID + "/members/" + usersID + "/activate"), null, function() {
      refreshDisplay("Personnel");
   });
}

function aoMemberCheck() {
   var user;
   var AOMember = false;
   $.ajax({
      async : false,
      type : 'GET',
      url : siteUrl.concat("_api/Web/CurrentUser/Groups"),
      headers : {
         "accept" : "application/json;odata=verbose"
      },
      dataType : 'json',
      success : function(json) {
         user = json.d.results;
         for (var i = 0; i < user.length; i++) {
            // TODO this should check for Facility Name the the user is submitting for.
            if ((user[i].Title.indexOf("AO Group") > -1) || (user[i].Title.indexOf("Administrative Officer") > -1)) {
               AOMember = true;
            }
         }
      }
   });
   return AOMember;
}

/* exported removeAttachment */
function removeAttachment(file, teamFolder) {
   var data = [];
   var response = {};

   // console.log("looking to remove: " + file.split("-RAMS-")[0]);

   response = {
      "questionsID" : file.split("-RAMS-")[0]
   };

   data.push(response);

   sendREST("POST", restPath.concat("study/" + assignedID + "/responses/" + targetStage), data, function() {
      sendREST("POST", siteUrl
         .concat("/_api/Web/GetFileByServerRelativeUrl('/sites/IRBApplications/Shared%20Documents/" + assignedID + "/"
            + teamFolder + "/" + file + "')"), null, function() {
         refreshDisplay("Files");
      }, "Delete");
   });
}

/* exported getAttachmentItem */
function getAttachmentItem(itemID) {

   // Send the request and return the response.
   return jQuery.ajax({
      url : siteUrl.concat("/_api/Web/Lists/getByTitle('Shared Documents')/Items(" + itemID + ")"),
      type : "GET",
      headers : {
         "accept" : "application/json;odata=verbose"
      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError((siteUrl.concat("/_api/Web/Lists/getByTitle('Shared Documents')/Items(" + itemID + ")")),
            errorThrown, textStatus, JSON.stringify(XMLHttpRequest));
      }
   });
}

/* exported updateAttachmentStatus */
function updateAttachmentStatus(itemMetadata, docStatus) {

   // Define the list item changes. Use the FileLeafRef property to change the display name.
   // For simplicity, also use the name as the title.
   // The example gets the list item type from the item's metadata, but you can also get it from the
   // ListItemEntityTypeFullName property of the list.
   var item;
   var approvedDate = new Date();
   if (docStatus == "Approved") {
      item = {
         "__metadata" : {
            "type" : itemMetadata.type
         },
         "Status" : docStatus,
         "DateApproved" : approvedDate
      };
   } else {

      item = {
         "__metadata" : {
            "type" : itemMetadata.type
         },
         "Status" : docStatus
      };
   }

   // Send the request and return the promise.
   // This call does not return response content from the server.
   return jQuery.ajax({
      url : itemMetadata.uri,
      type : "POST",
      data : JSON.stringify(item),
      headers : {
         "X-RequestDigest" : requestDigest,
         "content-type" : "application/json;odata=verbose",
         "IF-MATCH" : itemMetadata.etag,
         "X-HTTP-Method" : "MERGE"
      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError((itemMetadata.uri), errorThrown, textStatus, JSON.stringify(XMLHttpRequest));
      }
   });
}
/* exported updateAttachment */
function updateAttachment() {
   /* jshint unused: false */
   var docStatus = $('#status').find(':selected').val();
   var attachmentItem = getAttachmentItem(currentAttachmentID);
   attachmentItem.done(function(listItem, status, xhr) {

      // Change the display name and title of the list item.
      var changeItem = updateAttachmentStatus(listItem.d.__metadata, docStatus);
      changeItem.done(function(data, status, xhr) {
         alert('Attachment status updated');
      });
      changeItem.fail(function(data, status, xhr) {
         alert('File upload failed: ' + status);
      });
   });
   attachmentItem.fail(function(data, status, xhr) {
      alert('Get attachment failed: ' + status);
   });

}

/* exported rerouteApp */
function rerouteApp(childId) {
   window.location.href("irb-applications-general.html?" + childId);
}

/**
 * @param subscriptionId {string} Workflow subscription id
 * @param itemId {number} list item id for which to start workflow. If site workflow, then send null for itemId
 * @param usersGroup {string} the users to send the email notification to.
 */
// Subscription id - Workflow subscription id
// list item id for which to start workflow. If site workflow, then send null for itemId
function startRAMSWorkflow(subscriptionId, itemId, usersGroup) {
   var ctx = SP.ClientContext.get_current();
   var wfManager = SP.WorkflowServices.WorkflowServicesManager.newObject(ctx, ctx.get_web());
   var subscription = wfManager.getWorkflowSubscriptionService().getSubscription(subscriptionId);
   ctx.load(subscription, 'PropertyDefinitions');
   ctx.executeQueryAsync(function(sender, args) {
      /* jshint unused: false */
      var params = {};
      // Find initiation data to be passed to workflow.
      var formData = subscription.get_propertyDefinitions().FormData;
      if ((formData !== null) && (typeof formData !== "undefined") && (formData !== "")) {
         var assocParams = formData.split(";#");
         for (var i = 0; i < assocParams.length; i++) {
            params[assocParams[i]] = subscription.get_propertyDefinitions()[assocParams[i]];
         }
      }
      if (itemId) {
         // guid is not defined. using subscription instead
         // wfManager.getWorkflowInstanceService().startWorkflowOnListItem(guid, itemId, params);
         wfManager.getWorkflowInstanceService().startWorkflowOnListItem(subscription, itemId, params);
      } else {
         params = {
            Study_ID : assignedID,
            To : usersGroup,
            Subject : "Personnel changed",
            Body : "The personnel changed."

         };
         wfManager.getWorkflowInstanceService().startWorkflow(subscription, params);
      }
      ctx.executeQueryAsync(function(sender, args) {
      }, function(sender, args) {
         alert('Failed to run workflow');
      });
   }, function(sender, args) {
      /* jshint unused: false */
      alert('Failed to run workflow');
   });
}
// TODO TAR The following function is commented out in dev, but is also available in DynamicGeneration or RAMS_Tools

// function getFacilityNameById(id) {
// var facilityJson;
// var facilityName;
// $.ajax({
// async: false,
// type: 'GET',
// url: siteUrl.concat("_api/Web/lists/getbytitle('Facility')/items" + "?$filter=Id eq '" + id + "'"),
// headers: {"accept": "application/json;odata=verbose"},
// dataType: 'json',
// success: function(json){
// facilityJson= json.d.results;
// facilityName=facilityJson[0].FacilityName;
//                        
// }
// });
// return facilityName;
// }

