/**
 * @fileOverview rams-environment provides common environment variables used by many pages and scripts.
 * @version 2.3.0
 */

// JSHINT {var} is defined but never used.
// jshint -W098
var protocol = "https://";
var port = "PORT";

var reportsServerName = "";
var mySitesServerName = "";
var consoleLoggingEnabled = false;

var RAMSVersion = "2.3.0";
var RAMSCopyright = " &copy; 2016-2018 VA RAMS (v" + RAMSVersion + ") ";
var RAMSApp = "RAMS-" + RAMSVersion;
var env = window.location.hostname;
var siteCollection = "/sites/IRBApplications/";

var dynamicSharepointURL = "";
var dynamicWebServiceURL = "";

var dynamicReportURL = "";
var RAMS = {};
var Roles =  [ "Investigator",
               "Co-Investigator", "Sub-Investigator", "Biostatistician", "Clinical Coordinator", "Collaborator",
               "CSP Program Manager", "Data Manager", "Lab Manager", "Lab Technician", "National Study Coordinator",
               "Nurse Assessor", "Pharmacist", "Pharmacy Technician", "Regulatory Coordinator", "Research Assistant",
               "Research Nurse", "Statistician", "Study Coordinator"];


if ((env.indexOf("idev") !== -1) || (env.indexOf("itst") !== -1)) {
   protocol = "https://";
   port = "8888";
   dynamicSharepointURL = protocol + env + siteCollection;
   dynamicWebServiceURL = protocol + env + ":" + port + "/" + RAMSApp + "/";
} else {
   dynamicSharepointURL = protocol + env + siteCollection;
   dynamicWebServiceURL = protocol + env + ":" + port + "/" + RAMSApp + "/";
}

if (env.indexOf("idev") !== -1) {
   reportsServerName = "URL;
   mySitesServerName = protocol + "DNS";
   consoleLoggingEnabled = true;
} else if (env.indexOf("itst") !== -1) {
   reportsServerName = "DNS";
   mySitesServerName = protocol + "DNS";
   consoleLoggingEnabled = true;
} else if (env.indexOf("sqa") !== -1) {
   reportsServerName = "DNS";
   mySitesServerName = protocol + "DNS";
   consoleLoggingEnabled = true;
} else if (env.indexOf("ppd") !== -1) {
   reportsServerName = "DNS";
   mySitesServerName = protocol + "DNS";
} else {
   reportsServerName = "DNS";
   mySitesServerName = protocol + "DNS";
}

dynamicReportURL = protocol + reportsServerName + "/Reports/Pages/Folder.aspx?ItemPath=%2fRAMS+Reports&ViewMode=List";

// Define environmental constants for RAMS web-apps.
window.RAMS = window.RAMS || {};
RAMS.Environment = {
   SharePoint : {
      URL : dynamicSharepointURL,
      CentralIRBSite : "Central IRB",
      List : {}
   // Column mappings defined below.
   },
   WebService : {
      URL : dynamicWebServiceURL
   // Form mappings defined below.
   },
   Reports : {
      URL : dynamicReportURL
   },
   cirbReports : {
      URL : dynamicSharepointURL + "SiteAssets/CIRB_Reports.html"
   },

   TMS : {
      tmsusername : "AI",
      tmspwd : "AI"
   }

};

// ////////////////
// REPLACE HERE //
// ////////////////
RAMS.Environment.SharePoint.List.Application = {
   FacilitiesLookupId : "FacilitiesLookupId",
   SiteLookupId : "SiteLookupId",
   LinkToApplication : "LinktoApplication",
   StudyId : "Study_ID",
   PrincipalInvestigatorId : "PrincipalInvestigatorId",
   CoPrincipalInvestigatorId : "CoPIId",
   PrimaryReviewersId : "PrimaryReviewersId",
   LocalSiteLiaisonsId : "ParticipatingSiteLiaisonId",
   ExpeditedCategories : [ "Drugs/Devices without need for IND/IDE", "Minimal draw of blood samples",
      "Non-invasively collected biological specimens", "Non-invasively collected data from clinical practice",
      "Data/documents collected for non-research", "Voice/video/image for research",
      "Behavioral characteristics or survey/interview/program evaluation/QA"
   ],
   ExemptCategories : [ "Educational Settings", "Educational Tests", "Public Officials", "Existing Data",
      "Demonstration", "Food"
   ],
   OffsiteWaiverRequests : "OffsiteWaiverRequests",
   AdverseReportableEvent : "AdverseReportableEvent",
   DeviationReportableEvent : "DeviationReportableEvent",
   IACUCReviewRequired : "IACUCReviewRequired",
   IRBReviewRequired : "IRBReviewRequired",
   RDCReviewRequired : "RDCReviewRequired",
   SRSReviewRequired : "SRSReviewRequired",
   SC1ReviewRequired : "SC1ReviewRequired",
   SC2ReviewRequired : "SC2ReviewRequired",
   SC3ReviewRequired : "SC3ReviewRequired"
};

RAMS.Environment.SharePoint.List.CommitteeStatus = {
   APPROVED: 1,
   APPROVEDWITHMAJORMODIFICATIONS: 2,
   APPROVEDWITHMINORMODIFICATIONS: 3,
   AWAITINGREVISIONS: 4,
   CLOSED: 5,
   DISAPPROVED: 6,
   DRAFT: 7,
   SUBMITTED: 8,
   SUSPENDED: 9,
   UNDERREVIEW: 10,
   APPROVEDCHOICE: "Approved",
   APPROVEDWITHMAJORMODIFICATIONSCHOICE: "Approved with Major Modifications",
   APPROVEDWITHMINORMODIFICATIONSCHOICE: "Approved with Minor Modifications",
   AWAITINGREVISIONSCHOICE: "Awaiting Revisions",
   CLOSEDCHOICE: "Closed",
   DISAPPROVEDCHOICE: "Disapproved",
   DRAFTCHOICE: "Draft",
   SUBMITTEDCHOICE: "Submitted",
   SUSPENDEDCHOICE: "Suspended",
   UNDERREVIEWCHOICE: "Under Review"
};

// The RAMS.Environment.WebService.Form represents the current active forms used by the RAMS application. These are the
// forms that will be created when submitting a new study application or creating a child application.
RAMS.Environment.WebService.Form = {
   Parent : 59,
   59 : "Parent",
   Local : 60,
   60 : "Local",
   Adverse : 61,
   61 : "Adverse",
   ContReview : 62,
   62 : "ContReview",
   Closeout : 63,
   63 : "Closeout",
   AdministrativeUpdate : 64,
   64 : "AdministrativeUpdate"
};

// ////////////////
// REPLACE HERE //
// ////////////////

/**
 * This is a common function used to sanitize data entered by users on HTML pages. It makes sure that data entered by
 * the user that has "<" or ">" symbols, indicating a "<script>" or html, is properly escaped so that it is not
 * executable.
 * 
 * @param {json} the json string to be sanitized. IT can contain any number of fields in the json string and all will be
 *           sanitized.
 */
function getFixedString(json) {
   var jsonString = "";

   if (typeof json !== "undefined") {
      jsonString = JSON.stringify(json, replacer);
      return JSON.parse(jsonString);
   } else {
      return json;
   }
}

/**
 * verifies that the val is a string and replaces "<" with "&lt;" and ">" with "&gt;"
 * @param name String the name if the field to be sanitized
 * @param val String the value to be sanitized and returned
 * @returns val String the sanitized value
 */
function replacer(name, val) {
   // convert javascript or html special characters to string
   if (typeof val === "string" && (val.indexOf("<") !== -1 || val.indexOf(">") !== -1)) {
      val = val.replace(/</g, "&lt;");
      val = val.replace(/>/g, "&gt;");
      return val;
   } else {
      return val; // return as is
   }
}

/**
 * used to display the RAMS version number from the version link in the footer. 
 */
function displayVersion() {
   alert("The RAMS application version is " + RAMSVersion + ".");
}
