$weburl = Get-SiteURL
$webSite = Get-SiteWeb

$Committees = @("IACUC", "IRB", "RDC", "SRS", "SC1", "SC2", "SC3")

$AdministratorDN = "Administrator;User"
$PrimaryReviewerDN = "Primary Reviewer;User"
$ReviewersDN = "Reviewers;User"
$SubmitDateDN = "Submit Date;Date"
$ApprovedDateDN = "Approved Date;Date"
$ExpiresDateDN = "Expires Period;Integer"
$TriennialReviewDateDN = "Triennial Review Period;Integer"
$StatusDN = "Status;Choice"
$ReviewRequiredDN = "Review Required;Boolean"

$ALColumns = @($AdministratorDN, $PrimaryReviewerDN, $ReviewersDN, $SubmitDateDN, $ApprovedDateDN, $ExpiresDateDN, $TriennialReviewDateDN, $StatusDN, $ReviewRequiredDN)

$separator = ";"
$option = [System.StringSplitOptions]::RemoveEmptyEntries

#CREATE SITE COLUMNS
Write-Host "CREATING SITE COLUMNS AND ADDING THEM TO CONTENT TYPE" -ForegroundColor Green

foreach($com in $Committees) {
    foreach($column in $ALColumns) {
        $scDN,$scType = $column.split($separator,$option)
        $scDN = $com + " " + $scDN
        $scIN = $scDN.replace(" ","")
        if($scType -eq "Choice") {
            Set-SiteColumnLookup -colName $scDN -colStaticName $scIN -lookupListName "CommitteeStatus" -lookupColumnName "Title" 
        } else {
            Set-SiteColumn -web $webSite -sFieldDisplayName $scDN -sFieldStaticName $scIN -sFieldInternalName $scIN -sFieldType $scType  -spFieldGroup "RAMS IRB" -showInEditForm -showInNewForm
        }
        Add-SiteColumnToCT -web $webSite -ctName "Application CT" -sFieldInternalName $scIN -sFieldStaticName $scIN
        Write-Host "Created site column $scDN, $scIN with type:$scType to RAMS IRB and added it to Application CT" -ForegroundColor Green
    }
}
Add-SiteColumnToCT -web $webSite -ctName "Application CT" -sFieldInternalName "FacilityLookup" -sFieldStaticName "FacilityLookup"

#Add Triennial Review Date
Set-SiteColumn -web $webSite -sFieldDisplayName "Triennial Review  Date" -sFieldStaticName "TriennialReviewDate" -sFieldInternalName "TriennialReviewDate" -sFieldType DateTime  -spFieldGroup "RAMS IRB" -showInEditForm -showInNewForm
Add-SiteColumnToCT -web $webSite -ctName "Application CT" -sFieldInternalName "TriennialReviewDate" -sFieldStaticName "TriennialReviewDate"

$webSite.Update()
