Add-PSSnapin Microsoft.SharePoint.PowerShell

if ((get-module | Where-Object({$_.Name -eq 'RAMS_Module_Util'}) ) -eq $null) {
    Import-Module .\RAMS_Module_Util.psm1 -Verbose
}

$siteWeb = Get-SiteWeb

#add New security Groups
function New-Group ($groupN) {
	$facilitylist =$w.Lists["Facility"]
	$facilitylistitems = $facilitylist.items
	#Loop through List
	#Get Facility Lookup
	#Copy record to correct list
	foreach ($item in $facilitylistitems) {
		$_facilityName = $item["Title"]
		New-RAMSGroup -facility $_facilityName -newGroupName $groupN -newGroupPermission "Edit" -groupOwnerSuffix "Local Site Administrators" -newGroupDescSuffix "Animal Facility Managers Group"
		Write-Host " $_facilityName $groupN has been added" -ForegroundColor Green	
	}
}


function Add-AnimalFacilityManagerGroups {
    #add new fields to the facility list to store new security groups
    #Animal Facility ManagersGroup
    New-Group -groupN  "Animal Facility Managers"
    Set-SiteColumn -web $siteWeb -sFieldDisplayName "Animal Facility Managers" -sFieldInternalName AnimalFacilityManagers -sFieldType User -sChoice AllUsers   -spFieldGroup "Animal Facility Managers Group"
    Add-SiteColumnToCT -web $siteWeb -ctName "Facility CT" -sFieldInternalName AnimalFacilityManagers -sFieldStaticName AnimalFacilityManagers

    #update facility list
    $facilityList = $siteWeb.GetListFromUrl("Lists/Facility/AllItems.aspx")
    #update the view to include two new fields
    $view = $facilityList.Views | ? { $_.Title -eq 'All items' }
    $view.ViewFields.Add("AnimalFacilityManagers");
    $view.Update();

    #populate new fields, for each facility (per row) w/ new relevant security groups.
    foreach ($item in $facilityList.Items) {
        #facility name
        $facilityName = $item["Facility Name"]
        #populate the value for new groups
        $item["Animal Facility Managers"] = $siteWeb.SiteGroups["$facilityName Animal Facility Managers"]
        #update
        $item.Update()
    }
}

Add-AnimalFacilityManagerGroups

