Add-PSSnapin Microsoft.SharePoint.PowerShell

if ((get-module | Where-Object({$_.Name -eq 'RAMS_Module_Util'}) ) -eq $null) {
    Import-Module .\RAMS_Module_Util.psm1 -Verbose
}
$weburl = Get-SiteURL
$webSite = Get-SPWeb $weburl

#CREATE SITE COLUMNS
Write-Host "CREATING SITE COLUMNS" -ForegroundColor Green

$facilityNameDN = "Facility Name"
$facilityNameIN = "FacilityName"
$facilitySiteCodeDN = "Site Code"
$facilitySiteCodeIN = "SiteCode"
$facilityLocationDN = "Facility Location"
$facilityLocationIN = "FacilityLocation"
$facilitySubSiteUrlDN = "Facility Subsite"
$facilitySubSiteUrlIN = "FacilitySubsite"
$facilityLiaisonDN = "Facility Liaison"
$facilityLiaisonIN = "FacilityLiaison"
# Local Site Administrators is a Display Name Change from Facility Administrators
$facilityLocalSiteAdministratorDN = "Local Site Administrators"
$facilityLocalSiteAdministratorIN = "LocalSiteAdministrators"
$facilityLocalSiteLiaisonsDN = "Local Site Liaisons"
$facilityLocalSiteLiaisonsIN = "LocalSiteLiaisons"
# Should Administrative Officers. Research Compliance Officers, and Animal Facility Managers be in the RAMS IRB Group?
$facilityAdministrativeOfficersDN = "Administrative Officers"
$facilityAdministrativeOfficersIN = "AdministrativeOfficers"
$facilityResearchComplianceOfficersDN = "Research Compliance Officers"
$facilityResearchComplianceOfficersIN = "ResearchComplianceOfficers"
$facilityAnimalFacilityManagersDN = "Animal Facility Managers"
$facilityAnimalFacilityManagersIN = "AnimalFacilityManagers"

$facilityContentType = "Facilities CT"

$facilitiesListNameDN = "Facilities"
$facilitiesListNameIN = "Facilities"

# TODO - CHITA - verify fieldTypes 
Set-SiteColumn -web $webSite -sFieldDisplayName $facilityNameDN -sFieldStaticName $facilityNameIN -sFieldInternalName $facilityNameIN -sFieldType "Text"  -spFieldGroup "RAMS IRB" 
Set-SiteColumn -web $webSite -sFieldDisplayName $facilityLocationDN -sFieldStaticName $facilityLocationIN -sFieldInternalName $facilityLocationIN -sFieldType "Text"  -spFieldGroup "RAMS IRB" 
Set-SiteColumn -web $webSite -sFieldDisplayName $facilitySiteCodeDN -sFieldStaticName $facilitySiteCodeIN -sFieldInternalName $facilitySiteCodeIN -sFieldType "Integer"  -spFieldGroup "RAMS IRB" 
Set-SiteColumn -web $webSite -sFieldDisplayName $facilitySubSiteDN -sFieldStaticName $facilitySubSiteIN -sFieldInternalName $facilitySubSiteIN -sFieldType "Hyperlink"  -spFieldGroup "RAMS IRB" 
Set-SiteColumn -web $webSite -sFieldDisplayName $facilityLiaisonDN -sFieldStaticName $facilityLiaisonIN -sFieldInternalName $facilityLiaisonIN -sFieldType "User"  -spFieldGroup "RAMS IRB" 
Set-SiteColumn -web $webSite -sFieldDisplayName $facilityLocalSiteAdministratorDN -sFieldStaticName $facilityLocalSiteAdministratorIN -sFieldInternalName $facilityLocalSiteAdministratorIN -sFieldType "User"  -spFieldGroup "RAMS IRB" 
Set-SiteColumn -web $webSite -sFieldDisplayName $facilityLocalSiteLiaisonsDN -sFieldStaticName $facilityLocalSiteLiaisonsIN -sFieldInternalName $facilityLocalSiteLiaisonsIN -sFieldType "User"  -spFieldGroup "RAMS IRB" 
Set-SiteColumn -web $webSite -sFieldDisplayName $facilityAdministrativeOfficersDN -sFieldStaticName $facilityAdministrativeOfficersIN -sFieldInternalName $facilityAdministrativeOfficersIN -sFieldType "User"  -spFieldGroup "RAMS IRB" 
Set-SiteColumn -web $webSite -sFieldDisplayName $facilityResearchComplianceOfficersDN -sFieldStaticName $facilityResearchComplianceOfficersIN -sFieldInternalName $facilityResearchComplianceOfficersIN -sFieldType "User"  -spFieldGroup "RAMS IRB" 
Set-SiteColumn -web $webSite -sFieldDisplayName $facilityAnimalFacilityManagersDN -sFieldStaticName $facilityAnimalFacilityManagersIN -sFieldInternalName $facilityAnimalFacilityManagersIN -sFieldType "User"  -spFieldGroup "RAMS IRB" 

#ADD SITE COLUMNS TO CONTENT TYPE
Write-Host "ADDING SITECOLUMNS TO CONTENT TYPE" -ForegroundColor Green
# TODO - CHITA - Need to add FieldRequired attribute, default to false if not provided.
Add-SiteColumnToCT -web $webSite -ctName $facilityContentType -sFieldInternalName $facilityNameIN -sFieldStaticName $facilityNameIN -sFieldRequired $true
Add-SiteColumnToCT -web $webSite -ctName $facilityContentType -sFieldInternalName $facilityLocationIN -sFieldStaticName $facilityLocationIN -sFieldRequired $false
Add-SiteColumnToCT -web $webSite -ctName $facilityContentType -sFieldInternalName $facilitySiteCodeIN -sFieldStaticName $facilitySiteCodeIN -sFieldRequired $false
Add-SiteColumnToCT -web $webSite -ctName $facilityContentType -sFieldInternalName $facilitySubSiteIN -sFieldStaticName $facilitySubSiteIN -sFieldRequired $true
Add-SiteColumnToCT -web $webSite -ctName $facilityContentType -sFieldInternalName $facilityLiaisonIN -sFieldStaticName $facilityLiaisonIN -sFieldRequired $true
Add-SiteColumnToCT -web $webSite -ctName $facilityContentType -sFieldInternalName $facilityLocalSiteAdministratorIN -sFieldStaticName $facilityLocalSiteAdministratorIN -sFieldRequired $true
Add-SiteColumnToCT -web $webSite -ctName $facilityContentType -sFieldInternalName $facilityLocalSiteLiaisonsIN -sFieldStaticName $facilityLocalSiteLiaisonsIN -sFieldRequired $false
Add-SiteColumnToCT -web $webSite -ctName $facilityContentType -sFieldInternalName $facilityAdministrativeOfficersIN -sFieldStaticName $facilityAdministrativeOfficersIN -sFieldRequired $true
Add-SiteColumnToCT -web $webSite -ctName $facilityContentType -sFieldInternalName $facilityResearchComplianceOfficersIN -sFieldStaticName $facilityResearchComplianceOfficersIN -sFieldRequired $true
Add-SiteColumnToCT -web $webSite -ctName $facilityContentType -sFieldInternalName $facilityAnimalFacilityManagersIN -sFieldStaticName $facilityAnimalFacilityManagersIN -sFieldRequired $true

#ADD SITE COLUMNS TO CONTENT TYPE
Write-Host "CREATING FACILITIES LIST" -ForegroundColor Green
Add-List -web $webSite 
# ID, Target Audience, Created, Created By, Modified, and Modified By are standard SP columns
Add-SiteColumnToList -web $webSite -sFieldInternalName "ID"
Add-SiteColumnToList -web $webSite -sFieldInternalName $facilityNameIN
Add-SiteColumnToList -web $webSite -sFieldInternalName $facilityLocationIN
Add-SiteColumnToList -web $webSite -sFieldInternalName $facilitySiteCodeIN
Add-SiteColumnToList -web $webSite -sFieldInternalName $facilitySubSiteIN
Add-SiteColumnToList -web $webSite -sFieldInternalName $facilityLiaisonIN
Add-SiteColumnToList -web $webSite -sFieldInternalName $facilityLocalSiteAdministratorIN
Add-SiteColumnToList -web $webSite -sFieldInternalName $facilityLocalSiteLiaisonsIN
Add-SiteColumnToList -web $webSite -sFieldInternalName $facilityAdministrativeOfficersIN
Add-SiteColumnToList -web $webSite -sFieldInternalName $facilityResearchComplianceOfficersIN
Add-SiteColumnToList -web $webSite -sFieldInternalName $facilityAnimalFacilityManagersIN
Add-SiteColumnToList -web $webSite -sFieldInternalName "Target Audiences"
Add-SiteColumnToList -web $webSite -sFieldInternalName "Created"
Add-SiteColumnToList -web $webSite -sFieldInternalName "Created By"
Add-SiteColumnToList -web $webSite -sFieldInternalName "Modified"
Add-SiteColumnToList -web $webSite -sFieldInternalName "Modified By"

#Create List Views and Detail Forms
Write-Host "CREATING FACILITIES LIST VIEW" -ForegroundColor Green
# TODO - CHITA - Create All Items View in field order shown above

Write-Host "CREATING FACILITIES NEW DETAIL FORM" -ForegroundColor Green
# TODO - CHITA - Create new Forms in field order shown above

Write-Host "CREATING FACILITIES EDIT DETAIL FORM" -ForegroundColor Green

Write-Host "CREATING FACILITIES DISPLAY DETAIL FORM" -ForegroundColor Green


#clean
if($webSite -ne $null) {
    $webSite.Dispose();
}