
    #check to ensure Microsoft.SharePoint.PowerShell is loaded
    $snapin = Get-PSSnapin | Where-Object {$_.Name -eq 'Microsoft.SharePoint.Powershell'}
    if ($snapin -eq $null)
    {
       Add-PSSnapin "Microsoft.SharePoint.Powershell"
    }


function UploadFiles($siteUrl, $DestFolderUrl, $LocalFileOrFolderPath)
{
   if((get-item $LocalFileOrFolderPath).PSIsContainer -eq $true)
   {
     write-host "Looping Items in Folder: $LocalFileOrFolderPath";
     Get-ChildItem $LocalFileOrFolderPath | Foreach-Object{
                                                            $name = $_.Name
                                                            $fullName = $_.FullName

                                                            if((get-item $fullName).PSIsContainer -eq $false){ #only files, not folders
                                                                    UploadandApprove -siteUrl $siteUrl -DestFolderUrl $DestFolderUrl -LocalFilePath $fullName
                                                                }
                                                                else{
                                                                    createSPFolder -siteUrl $siteUrl -DestFolderUrl $DestFolderUrl -LocalFolderName $name | Out-Null
                                                                    UploadFiles -siteUrl $siteUrl -DestFolderUrl ($DestFolderUrl + "/" +$name) -LocalFileOrFolderPath $fullName #recursively iterate 
                                                                }
                                                         
                                                          }
     Write-Host "Loop completed";
   }
   else
   {
     UploadandApprove -siteUrl $siteUrl -DestFolderUrl $DestFolderUrl -LocalFilePath $_.FullName
   }
}

function createSPFolder($siteUrl,  $DestFolderUrl, $LocalFolderName)
{
    
    write-host "Creating SPFolder: " $LocalFolderName
    ############################################# Upload ###########################################
    $spSite = New-Object Microsoft.SharePoint.SPSite($siteUrl)
    $spWeb = $spSite.OpenWeb()  
    try    
    {  
        $destfolder= $spWeb.GetFolder($DestFolderUrl)
        $destfolder.SubFolders.Add($LocalFolderName)
        $destfolder.update();
    }
    catch
    {
        write-host -ForegroundColor Red "Exception Occurred:"
       Write-Host -ForegroundColor Red $_.Exception.Message
       Write-Host -ForegroundColor Red $_.Exception.stacktrace
    } 
    finally
    {
        $spSite.Dispose()
        $spWeb.Dispose()
    }
}

function UploadandApprove($siteUrl, $DestFolderUrl, $LocalFilePath)
{

    write-host "Uploading " $LocalFilePath
    ############################################# Upload ###########################################
    $spSite = New-Object Microsoft.SharePoint.SPSite($siteUrl)
    $spWeb = $spSite.OpenWeb()  
    try    
    {  
        $destfolder= $spWeb.GetFolder($DestFolderUrl) 
        $spList = $destfolder.Item.ParentList            
        $spFiles = $destfolder.Files;  
 
    
         $fileInfo = Get-Item $LocalFilePath
         $fileContent = Get-Content $fileInfo -encoding byte
         $fileName = $fileInfo.Name
         $ExistingFile=$spFiles[$fileName]
         if($ExistingFile.Exists -and $spList.EnableVersioning -eq $true)
         {
            #check out
            $ExistingFile.CheckOut()    
         }
 
         #update file
         $spFile = $spFiles.Add($fileName,$fileContent,$true);
         
            if($spFile.CheckOutType  -ne [Microsoft.SharePoint.SPFile+SPCheckOutType]::None)
            {
                $spFile.CheckIn("")    
            }

            if($spList.EnableMinorVersions -eq $true)
            {
                $spFile.publish("");
            }
 
        $spSite.Dispose();
         #Dispose of Web object
        $spWeb.Dispose() 



        ########################### Approve Items ############################################

        $spSite = New-Object Microsoft.SharePoint.SPSite($siteUrl)
        $spWeb = $spSite.OpenWeb() 

        $destfolder= $spWeb.GetFolder($DestFolderUrl) 
        $spList = $destfolder.Item.ParentList    
        $spFiles = $destfolder.Files;  
 
         $fileInfo = Get-Item $LocalFilePath
         $fileContent = Get-Content $fileInfo -encoding byte
         $fileName = $fileInfo.Name
         $ExistingFile=$spFiles[$fileName]
         if($ExistingFile.Exists -and $spList.EnableModeration -eq $true)
         {
            $ExistingFile.Approve('')
    
         } 
         #Dispose of Web object
        $spWeb.Dispose()
    }
    catch
    {
        write-host -ForegroundColor Red "Exception Occurred:"
       Write-Host -ForegroundColor Red $_.Exception.Message
       Write-Host -ForegroundColor Red $_.Exception.stacktrace
    } 
    finally
    {
        $spSite.Dispose()
        $spWeb.Dispose()
    }
}



############# Begin here
#cls

write-host "DeployFiles.."

#$siteUrl = "http://vaww.itst.ram.msp.va.gov/sites/IRBApplications"

# The local path can be file or folder
# If folder, then all files will be iterated and uploaded.

$siteUrl = Get-SiteURL

UploadFiles -siteUrl $siteUrl  -DestFolderUrl ($siteUrl + "/_catalogs") -LocalFileOrFolderPath $fileLocationCatalog
UploadFiles -siteUrl $siteUrl -DestFolderUrl ($siteUrl + "/SiteAssets") -LocalFileOrFolderPath $fileLocationSiteAsset 
UploadFiles -siteUrl $siteUrl -DestFolderUrl ($siteUrl + "/SitePages") -LocalFileOrFolderPath  $fileLocationSitePages 
UploadFiles -siteUrl $siteUrl -DestFolderUrl ($siteUrl + "/PublishingImages") -LocalFileOrFolderPath  $fileLocationOnlineHelp 

