try {
Add-PSSnapin Microsoft.SharePoint.PowerShell -ErrorAction SilentlyContinue
}
catch {
 Write-Host "PSSnapin Microsoft.SharePoint.PowerShell already loaded" -ForegroundColor Yellow
}

if ((get-module | Where-Object({$_.Name -eq 'RAMS_Module_Util'}) ) -eq $null) {
    Import-Module .\RAMS_Module_Util.psm1 -Verbose
}

$weburl = Get-SiteURL
$webSite = Get-SiteWeb

#CREATING FACILITY SITE COLUMNS
Write-Host "CREATING FACILITY SITE COLUMNS" -ForegroundColor Green
#Set-SiteColumn -web $webSite -sFieldDisplayName "Local Site Liaisons" -sFieldStaticName "LocalSiteLiaisons" -sFieldInternalName "LocalSiteLiaisons" -sFieldType "User"  -spFieldGroup "RAMS IRB" 
#Set-SiteColumn -web $webSite -sFieldDisplayName "Local Site Administrators" -sFieldStaticName "LocalSiteAdministrators" -sFieldInternalName "LocalSiteAdministrators" -sFieldType "User" -spFieldGroup "RAMS IRB" 
$facilityList = Get-SPList -web $webSite -Name "Facility" -Url ($webSite.ServerRelativeUrl + "/Lists/Facility/AllItems.aspx")

#remove set of columns from Facility list that are part of Application_CT.
$webSite.ContentTypes | ? {$_.Name -match 'Application CT'} | % {    
    $appCTFields = $_.fields
    $facilityList.fields | ForEach-Object {
        $facilityFld = $_
        $appCTFields | ForEach-Object {
            if($_.StaticName -match $facilityFld.StaticName) {                
                if(($_.StaticName -notmatch "Title") -and ($_.StaticName -notmatch "ID") -and ($_.StaticName -notmatch "Hide") -and ($_.StaticName -notmatch "ContentType")) {   
                    Remove-ColumnFromList -WebURL (Get-SiteURL) -ListName "Facility" -ColumnName $_.Title
                    Write-Host $_.StaticName has been removed from $facilityList.Title -ForegroundColor Yellow
                }
        }
    }
 } 
}


$field = ($facilityList).Fields.GetFieldByInternalName("LocalSiteAdministrators")
$field.Title =  "Local Site Administrators"
$field.Update()
write-host "Site Column LocalSiteAdministrators is renamed from Facility Administrators to Local Site Administrators." -ForegroundColor Green

#ADDING SITECOLUMNS TO CONTENT TYPE
Write-Host "ADDING SITECOLUMNS TO FACILITY CONTENT TYPE" -ForegroundColor Green
Add-SiteColumnToCT -web $webSite -ctName "Facility CT" -sFieldInternalName "LocalSiteLiaisons" -sFieldStaticName "LocalSiteLiaisons"
Add-SiteColumnToCT -web $webSite -ctName "Facility CT" -sFieldInternalName "LocalSiteAdministrators" -sFieldStaticName "LocalSiteAdministrators"

# Does the views for the Facility List need to be updated?
$fldList = @(
    'LocalSiteLiaisons';'LocalSiteAdministrators'
)
#add new code for re-usable view on 03/17
Update-SpListView -fldList $fldList -listName 'Facility'

# Remove ApplicationCT and Committee CT from Facility List
# 03/17/2018: Tom; both Facility and Application list data needs to be fully cleaned/removed before remoing any content type, please add a new task for me in RTC, thanks! [Chita]

#Read more: http://www.sharepointdiary.com/2016/09/sharepoint-online-remove-content-type-from-list-using-powershell.html#ixzz5A2kgOjlQ
#Remove-ContentTypeFromList -SiteURL (Get-SiteURL) -ListName "Facility" -ContentTypeName "Facility CT" -Verbose
#Remove-ListContentType -web $webSite -ctName "Application CT" -listName "Facility" 
#Remove-ListContentType -web $webSite -ctName "Committee CT" -listName "Facility" 

#CREATING COMMITTEE SITE COLUMNS
Write-Host "CREATING COMMITTEE SITE COLUMNS" -ForegroundColor Green
$newDelegatedCommiteeList = @('IACUC','IRB','RDC','SRS')
foreach($newDelegatedCommitee in $newDelegatedCommiteeList) {

    #The Delegated fields below are Facility Lookup fields the value will be null or the Facility the Committee is delegated to.
    Set-SiteColumnLookup -web $webSite -sFieldDisplayName "$newDelegatedCommitee Delegated" -lookupListTitle "Facility" -spFieldGroup "RAMS IRB"

    #The Hidden fields below are Facility Lookup fields the value will be null or the Facility the Committee is delegated to.
    $fldTmp = $newDelegatedCommitee + 'Hidden'
    Set-SiteColumn -web $webSite -sFieldDisplayName "$newDelegatedCommitee Hidden" -sFieldStaticName $fldTmp -sFieldInternalName $fldTmp -sFieldType "Boolean"  -spFieldGroup "RAMS IRB" 

    #ADDING SITECOLUMNS TO CONTENT TYPE - No need to migrate any data for these fields.
    Write-Host "ADDING SITECOLUMNS TO COMMITTEE CONTENT TYPE" -ForegroundColor Green
    $fldTmp = $newDelegatedCommitee + ' Delegated'    
    Add-SiteColumnToCTByTitle -web $webSite -ctName "Committee CT" -sFieldTitle $fldTmp
    
    #Hidden
    $fldTmp = $newDelegatedCommitee + ' Hidden'    
    Add-SiteColumnToCTByTitle -web $webSite -ctName "Committee CT" -sFieldTitle $fldTmp
}


#Loop through Committee List and move the 
# Committee.LocalSiteLiaisons to Facility.LocalSiteLiaisons and 
# Committee.LocalSiteAdministrators to Facility.LocalSiteAdministrators
Write-Host "MOVING COMMITTEE LIST FIELDS TO FACILITY LIST" -ForegroundColor Green
#Read all the items in the Facility List
$facilityList = $webSite.Lists.TryGetList("Facility")
$committeeList = $webSite.Lists.TryGetList("Committee list")

foreach ($facility in $facilityList.Items) {
    foreach($commitee in $committeeList.Items) {
        if($commitee["Title"] -eq $facility["Facility Name"]) {
            $facility["LocalSiteAdministrators"] = $commitee["LocalSiteAdministrators"]
            $facility["LocalSiteLiaisons"] = $commitee["LocalSiteLiaisons"]
            write-host "FacilityList Local Site Administrator and Local Site Liaison for" $facility["Facility Name"] "updated." -ForegroundColor Green    
        }
    }
}


#Turn on versioning for the Facility List
$facilityList.EnableVersioning = $true
$facilityList.EnableMinorVersions = $false

$committeeList.EnableVersioning = $true
$committeeList.EnableMinorVersions = $false

$webSite.Update()

#clean
if($webSite -ne $null) {
    $webSite.Dispose();
}