﻿
Set-StrictMode -Version Latest

$ArrayList = New-Object System.Collections.ArrayList

function Get-ColumnValueFromAList ($Site, $subSite, $filterDate, $listUrl, $listName, $listColumnName)
{
    #get a collection of ids
    $itemsToDelete = New-Object System.Collections.ArrayList
    $list = Get-SPList -web $Site -Name $listName -Url $listUrl -subWeb $subSite
    #get application list
    if($list -ne $null)
    {
        #Get all items in this list and save them to a variable
        $items = $list.items
        $itemCount = 0
        #Go through all items
        Write-Host Total Items in the $list.EntityTypeName : $items.Count -ForegroundColor Green
        foreach($item in $items)
        {
            if($item["Created"] -lt $filterDate)
            { 
                #add each item id to the delete list  
                Write-Host Add item# $itemCount  from the list for Study#: $item["Title"] on $item["Created"] -ForegroundColor Cyan                                 
                $itemsToDelete.Add($item[$listColumnName])                
                $itemCount ++;
            }            
        }        
        #now start deleting those items
        if($itemCount -eq 0) {
            Write-Host Nothing to delete, count: $itemCount  -ForegroundColor Yellow
         return
        }
        Write-Host Total items to be deleted $itemCount  -ForegroundColor Yellow
    }
    else
    {
        Write-Host No list found by the name: $listUrl
    }
    return $itemsToDelete
}
function Remove-DataFromAListByColumn ($Site, $subSite, $ColumnSet, $listUrl, $listName, $listColumnName)
{
    #get a collection of ids
    $itemsToDelete = New-Object System.Collections.ArrayList
    $list = Get-SPList -web $Site -Name $listName -Url $listUrl -subWeb $subSite
    #get application list
    if($list -ne $null)
    {
        #Get all items in this list and save them to a variable
        $items = $list.items
        $itemCount = 0
        #Go through all items
        Write-Host Total Items in the $list.EntityTypeName : $items.Count -ForegroundColor Green
        foreach($item in $items)
        {
            foreach($col in $ColumnSet) {
                $x = $item.Url 
                if($x.IndexOf("/Study") -gt 0) {
                    $studyId = $x.Substring($x.IndexOf("/")+1,$x.IndexOf("/Study")-$x.IndexOf("/")-1)
                    if($studyId -eq $col)
                    { 
                        #add each item id to the delete list  
                        Write-Host Add item# $itemCount  from the list for Study#: $item["Title"] on $item["Created"] -ForegroundColor Cyan                                 
                        $itemsToDelete.Add($item.ID)                
                        $itemCount ++;
                    }           
                }
            }     
        }        
        #now start deleting those items
        if($itemCount -eq 0) {
            Write-Host Nothing to delete, count: $itemCount  -ForegroundColor Yellow
         return
        }
        Write-Host Total items to be deleted $itemCount  -ForegroundColor Yellow
        foreach($itemToDelete in $itemsToDelete) {
            Write-Host Deleting item# $itemToDelete -ForegroundColor Cyan  
            $items.DeleteItemById($itemToDelete);                                                                                                    
        }
        $items.Update();
        $list.Update();
        Write-Host Total Items After $list.items.Count  -ForegroundColor Green     
    }
    else
    {
        Write-Host No list found by the name: $listUrl
    }
    return $itemsToDelete
}

function Remove-RAMSHistoryData ($cleanDate, $dbServer)
{
    #get site collection
    #$site = Get-SPSite -Limit All | Select-String IRBApplications
    $site= Get-SiteURL
    $x = $site.ToString();
    $url = $x.Substring($x.IndexOf("=")+1)
    #$w = Get-SPWeb $url
    $w = Get-SiteWeb

    #filter by days
    $filterPriorDate = $cleanDate
    Write-Host "Delete data before $filterPriorDate" -ForegroundColor Magenta


    #Delete Application list for older studies.
    $listUrl = $url + "/Lists/Application/AllItems.aspx"
    $listOfStudies = Get-ColumnValueFromAList -Site $w -filterDate $filterPriorDate -listUrl $listUrl -listColumnName "Study_ID"

    Remove-DataFromAList -Site $w -filterDate $filterPriorDate -listUrl $listUrl

    #Delete workflow task list
    $listUrl = $url + "/WorkflowTasks/AllItems.aspx"
    Remove-DataFromAList -Site $w -filterDate $filterPriorDate -listUrl $listUrl

    #Delete Shared Docs for the same set of studies
    $listUrl = $url + "/Shared%20Documents/Forms/AllItems.aspx"
    Remove-DataFromAList -Site $w -filterDate $filterPriorDate -listUrl $listUrl
    Remove-DataFromAListByColumn -Site $w -ColumnSet $listOfStudies -listUrl $listUrl -listColumnName "Name"

    #delete items from list at sub-site level
    $listBucket = @(
    "FullBoardReviewCalendar",
    "CommitteeMinutes",
    "DataSharingRepositoryAgreements",

    "IACUC Calendar",

    "RDC Calendar",

    "SRSCalendar"
    );
  
  $formBucket = @(
    "IACUCDocuments",

    "RDCAgenda",
    "RDCDocuments"

    "RCOChecklists"
    );

    foreach($sub in $w.Webs) {
        #Delete list data
        foreach($listEgg in $listBucket) {
            $listUrl = $sub.url + "/Lists/" + $listEgg + "/AllItems.aspx"
            Remove-DataFromAList -filterDate $filterPriorDate -listName $listEgg -listUrl $listUrl -subSite $sub
        }
        #Delete form data
        foreach($formEgg in $formBucket) {
            $listUrl = $sub.url + "/" + $formEgg + "/Forms/AllItems.aspx"
            Remove-DataFromAList -filterDate $filterPriorDate -listName $formEgg -listUrl $listUrl -subSite $sub
        }
    }
    #clean from sql server    
    $recAffected =  RunSQL-ReturnInt -sqlQuery "exec DataCleanup @cutoffdate='$cleandate'" -dbServer $dbServer   
    Write-Host "Total $recAffected Studies have been deleted from the RAMS user database:" -ForegroundColor Yellow
}

Export-ModuleMember Remove-RAMSHistoryData

<#
e.g. Remove-RAMSHistoryData -cleanDate "12/30/2015 12:56:20" -dbServer DNS  

New-ModuleManifest -Path .\RAMS_Module_CleanHistory.psd1 -ModuleVersion 1.0.0 -Author "Chita Nilak, ByLight" -RootModule .\RAMS_Module_CleanHistory.psm1 -Description "RAMS Utility Functions" -CompanyName "VA.GOV.RAMS" -Verbose 
cd..
if((Get-Module -Name RAMS_Module_CleanHistory) -ne $null) {
    Remove-Module -Name RAMS_Module_CleanHistory
}
try {
    Import-Module .\RAMS_Module_CleanHistory.psd1 
}
catch {
    Write-Host Host: $env:COMPUTERNAME is not a Server, SharePoint is not installed on this computer...ignore, information only.
    #do nothing.
}

#>